/*****************************************************************************
 *
 *   z8001ops_seg.c
 *   Copyright Matt Knoth 2008 (p8000@knothusa.net)
 *
 *   based on Juergen Buchmueller's Z8002 MAME model
 *
 *   Portable Z8001 emulator
 *   Opcode functions
 *
 *   Z8002: Copyright Juergen Buchmueller, all rights reserved.
 *   Bug fixes and MSB_FIRST compliance Ernesto Corvi.
 *
 *   - This source code is released as freeware for non-commercial purposes.
 *   - You are free to use and redistribute this code in modified or
 *     unmodified form, provided you list me in the credits.
 *   - If you modify this source code, you must add a notice to each modified
 *     source file that it has been changed.  If you're a nice person, you
 *     will clearly mark each change too.  :)
 *   - If you wish to use this for commercial purposes, please contact me at
 *     pullmoll@t-online.de
 *   - The author of this copywritten work reserves the right to change the
 *     terms of its usage and license at any time, including retroactively
 *   - This entire notice must remain in the source code.
 *
 *****************************************************************************/

// FIXME FIXME -- mainly extended and cop instructions
static void Z4E_ssN0_dddd_segmented( void ) { }
static void Z4E_ssss_dddd_segmented( void ) { }
static void Z4F_ssN0_dddd_segmented( void ) { }
static void Z4F_ssss_dddd_segmented( void ) { }

#define GET_SEGADDR( x )   \
   UINT32 seg_addr;        \
   if(Z.op[x] & 0x8000) {  \
      Z.op[x+1] = RDOP(PC); PC += 2; \
      seg_addr = ((Z.op[x] & 0x7f00) << 16) | Z.op[x+1];         \
   } else {                                                      \
      seg_addr = ((Z.op[x] & 0x7f00)<<16) | (Z.op[x] & 0x00ff);  \
   }

#define GET_SEGADDR_IMM8( x )  \
   UINT8 imm8;                 \
   UINT32 seg_addr;            \
   if(Z.op[x] & 0x8000) {      \
      seg_addr = ((Z.op[x] & 0x7f00) << 16) | Z.op[x+1];         \
      Z.op[x+2] = RDOP(PC);  PC += 2; \
      imm8 = (UINT8)Z.op[x+2];                                   \
   } else {                                                      \
      seg_addr = ((Z.op[x] & 0x7f00)<<16) | (Z.op[x] & 0x00ff);  \
      imm8 = (UINT8)Z.op[x+1];                                   \
   }

#define GET_SEGADDR_IMM16( x )  \
   UINT16 imm16;                \
   UINT32 seg_addr;             \
   if(Z.op[x] & 0x8000) {       \
      seg_addr = ((Z.op[x] & 0x7f00) << 16) | Z.op[x+1];         \
      Z.op[x+2] = RDOP(PC);  PC += 2; \
      imm16 = Z.op[x+2];                                         \
   } else {                                                      \
      seg_addr = ((Z.op[x] & 0x7f00)<<16) | (Z.op[x] & 0x00ff);  \
      imm16 = Z.op[x+1];                                         \
   }


INLINE void PUSHWSEG(UINT8 dst, UINT16 value)
{
    RW(dst+1) -= 2;
    WRMEMSEG_W( RL(dst), value );
}

INLINE UINT16 POPWSEG(UINT8 src)
{
        UINT16 result = RDMEMSEG_W( RL(src) );
    	RW(src+1) += 2;
        return result;
}

INLINE void PUSHLSEG(UINT8 dst, UINT32 value)
{
        RW(dst+1) -= 4;
        WRMEMSEG_L( RL(dst), value );
}

INLINE UINT32 POPLSEG(UINT8 src)
{
        UINT32 result = RDMEMSEG_L( RL(src) );
    	RW(src+1) += 4;
        return result;
}

/******************************************
 addb    rbd,@rs
 flags:  CZSVDH                                                     *
 ******************************************/
static void Z00_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	RB(dst) = ADDB( RB(dst), RDMEMSEG_B(RL(src)) );
}

/******************************************
 add     rd,@rs
 flags:  CZSV--                                                     *
 ******************************************/
static void Z01_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	RW(dst) = ADDW( RW(dst), RDMEMSEG_W(RL(src)) );
}

/******************************************
 subb    rbd,@rs
 flags:  CZSVDH                                                     *
 ******************************************/
static void Z02_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	RB(dst) = SUBB( RB(dst), RDMEMSEG_B(RL(src)) ); /* EHC */
}

/******************************************
 sub     rd,@rs
 flags:  CZSV--                                                     *
 ******************************************/
static void Z03_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	RW(dst) = SUBW( RW(dst), RDMEMSEG_W(RL(src)) );
}

/******************************************
 orb     rbd,@rs
 flags:  CZSP--                                                     *
 ******************************************/
static void Z04_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	RB(dst) = ORB( RB(dst), RDMEMSEG_B(RL(src)) );
}

/******************************************
 or      rd,@rs
 flags:  CZS---                                                     *
 ******************************************/
static void Z05_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	RW(dst) = ORW( RW(dst), RDMEMSEG_W(RL(src)) );
}

/******************************************
 andb    rbd,@rs
 flags:  -ZSP--                                                     *
 ******************************************/
static void Z06_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	RB(dst) = ANDB( RB(dst), RDMEMSEG_B(RL(src)) );
}

/******************************************
 and     rd,@rs
 flags:  -ZS---                                                     *
 ******************************************/
static void Z07_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	RW(dst) = ANDW( RW(dst), RDMEMSEG_W(RL(src)) );
}

/******************************************
 xorb    rbd,@rs
 flags:  -ZSP--                                                     *
 ******************************************/
static void Z08_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	RB(dst) = XORB( RB(dst), RDMEMSEG_B(RL(src)) );
}

/******************************************
 xor     rd,@rs
 flags:  -ZS---                                                     *
 ******************************************/
static void Z09_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	RW(dst) = XORW( RW(dst), RDMEMSEG_W(RL(src)) );
}

/******************************************
 cpb     rbd,@rs
 flags:  CZSV--                                                     *
 ******************************************/
static void Z0A_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	CPB( RB(dst), RDMEMSEG_B(RL(src)) );
}

/******************************************
 cp      rd,@rs
 flags:  CZSV--                                                     *
 ******************************************/
static void Z0B_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	CPW( RW(dst), RDMEMSEG_W(RL(src)) );
}

/******************************************
 comb    @rd
 flags:  -ZSP--                                                     *
 ******************************************/
static void Z0C_ddN0_0000_segmented(void)
{
	GET_DST(OP0,NIB3);
	WRMEMSEG_B( RL(dst), COMB(RDMEMSEG_B(RL(dst))) );
}

/******************************************
 cpb     @rd,imm8
 flags:  CZSV--                                                     *
 ******************************************/
static void Z0C_ddN0_0001_imm8_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_IMM8(OP1);
  	CPB( RDMEMSEG_B(RL(dst)), imm8 ); 
//	CPB( RB(dst), imm8 );                                      // BUG !!!!!!
}

/******************************************
 negb    @rd
 flags:  CZSV--                                                     *
 ******************************************/
static void Z0C_ddN0_0010_segmented(void)
{
	GET_DST(OP0,NIB2);
	WRMEMSEG_B( RL(dst), NEGB(RDMEMSEG_B(RL(dst))) );
}

/******************************************
 testb   @rd
 flags:  -ZSP--                                                     *
 ******************************************/
static void Z0C_ddN0_0100_segmented(void)
{
	GET_DST(OP0,NIB2);
	TESTB(RDMEMSEG_B(RL(dst)));
}

/******************************************
 ldb     @rd,imm8
 flags:  ------                                                     *
 ******************************************/
static void Z0C_ddN0_0101_imm8_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_IMM8(OP1);
	WRMEMSEG_B( RL(dst), imm8 );
}

/******************************************
 tsetb   @rd
 flags:  --S---                                                     *
 ******************************************/
static void Z0C_ddN0_0110_segmented(void)
{
	GET_DST(OP0,NIB2);
    if (RDMEMSEG_B(RL(dst)) & S08) SET_S; else CLR_S;
    WRMEMSEG_B(RL(dst), 0xff);
}

/******************************************
 clrb    @rd
 flags:  ------                                                     *
 ******************************************/
static void Z0C_ddN0_1000_segmented(void)
{
	GET_DST(OP0,NIB2);
	WRMEMSEG_B( RL(dst), 0 );
}

/******************************************
 com     @rd
 flags:  -ZS---                                                     *
 ******************************************/
static void Z0D_ddN0_0000_segmented(void)
{
	GET_DST(OP0,NIB2);
	WRMEMSEG_W( RL(dst), COMW(RDMEMSEG_W(RL(dst))) );
}

/******************************************
 cp      @rd,imm16
 flags:  CZSV--                                                     *
 ******************************************/
static void Z0D_ddN0_0001_imm16_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_IMM16(OP1);
	CPW( RDMEMSEG_W(RL(dst)), imm16 );
}

/******************************************
 neg     @rd
 flags:  CZSV--                                                     *
 ******************************************/
static void Z0D_ddN0_0010_segmented(void)
{
	GET_DST(OP0,NIB2);
	WRMEMSEG_W( RL(dst), NEGW(RDMEMSEG_W(RL(dst))) );
}

/******************************************
 test    @rd
 flags:  -ZS---                                                     *
 ******************************************/
static void Z0D_ddN0_0100_segmented(void)
{
	GET_DST(OP0,NIB2);
	TESTW( RDMEMSEG_W(RL(dst)) );
}

/******************************************
 ld      @rd,imm16
 flags:  ------                                                     *
 ******************************************/
static void Z0D_ddN0_0101_imm16_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_IMM16(OP1);
	WRMEMSEG_W( RL(dst), imm16);
}

/******************************************
 tset    @rd
 flags:  --S---                                                     *
 ******************************************/
static void Z0D_ddN0_0110_segmented(void)
{
	GET_DST(OP0,NIB2);
    if (RDMEMSEG_W(RL(dst)) & S16) SET_S; else CLR_S;
    WRMEMSEG_W(RL(dst), 0xffff);
}

/******************************************
 clr     @rd
 flags:  ------                                                     *
 ******************************************/
static void Z0D_ddN0_1000_segmented(void)
{
	GET_DST(OP0,NIB2);
	WRMEMSEG_W( RL(dst), 0 );
}

/******************************************
 push    @rd,imm16
 flags:  ------                                                     *
 ******************************************/
static void Z0D_ddN0_1001_imm16_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_IMM16(OP1);
	PUSHWSEG( dst, imm16 );
}

/******************************************
 cpl     rrd,@rs
 flags:  CZSV--                                                     *
 ******************************************/
static void Z10_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	CPL( RL(dst), RDMEMSEG_L(RL(src)) );
}

/******************************************
 pushl   @rd,@rs
 flags:  ------                                                     *
 ******************************************/
static void Z11_ddN0_ssN0_segmented(void)
{
	GET_SRC(OP0,NIB3);
	GET_DST(OP0,NIB2);
	PUSHLSEG( dst, RDMEMSEG_L(RL(src)) );
}

/******************************************
 subl    rrd,@rs
 flags:  CZSV--                                                     *
 ******************************************/
static void Z12_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	RL(dst) = SUBL( RL(dst), RDMEMSEG_L(RL(src)) );
}

/******************************************
 push    @rd,@rs
 flags:  ------                                                     *
 ******************************************/
static void Z13_ddN0_ssN0_segmented(void)
{
	GET_SRC(OP0,NIB3);
	GET_DST(OP0,NIB2);
	PUSHWSEG( dst, RDMEMSEG_W(RL(src)) );
}

/******************************************
 ldl     rrd,@rs
 flags:  ------                                                     *
 ******************************************/
static void Z14_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	RL(dst) = RDMEMSEG_L( RL(src) );
}

/******************************************
 popl    @rd,@rs
 flags:  ------                                                     *
 ******************************************/
static void Z15_ssN0_ddN0_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	WRMEMSEG_L( RL(dst), POPLSEG( src ));
//	RL(dst) = POPLSEG( src );                           // BUG !!!!!
}

/******************************************
 addl    rrd,@rs
 flags:  CZSV--                                                     *
 ******************************************/
static void Z16_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	RL(dst) = ADDL( RL(dst), RDMEMSEG_L(RL(src)) );
}

/******************************************
 pop     @rd,@rs
 flags:  ------                                                     *
 ******************************************/
static void Z17_ssN0_ddN0_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	WRMEMSEG_W( RL(dst), POPWSEG( src ));
//	RW(dst) = POPWSEG( src );                        // BUG !!!!!
}

/******************************************
 multl   rqd,@rs
 flags:  CZSV--                                                     *
 ******************************************/
static void Z18_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	RQ(dst) = MULTL( RQ(dst), RDMEMSEG_L(RL(src)) );
//      RQ(dst) = MULTL( RQ(dst), RL(src) );            // BUG !!!!!!

}

/******************************************
 mult    rrd,@rs
 flags:  CZSV--                                                     *
 ******************************************/
static void Z19_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	RL(dst) = MULTW( RL(dst), RDMEMSEG_W(RL(src)) );
}

/******************************************
 divl    rqd,@rs
 flags:  CZSV--                                                     *
 ******************************************/
static void Z1A_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	RQ(dst) = DIVL( RQ(dst), RDMEMSEG_L(RL(src)) );
}

/******************************************
 div     rrd,@rs
 flags:  CZSV--                                                     *
 ******************************************/
static void Z1B_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	RL(dst) = DIVW( RL(dst), RDMEMSEG_W(RL(src)) );
}

/******************************************
 testl   @rd
 flags:  -ZS---                                                     *
 ******************************************/
static void Z1C_ddN0_1000_segmented(void)
{
	GET_DST(OP0,NIB2);
	TESTL( RDMEMSEG_L(RL(dst)) );
}

/******************************************
 ldm     @rd,rs,n
 flags:  ------                                                     *
 ******************************************/
static void Z1C_ddN0_1001_0000_ssss_0000_nmin1_segmented(void)
{
    GET_DST(OP0,NIB2);
    GET_CNT(OP1,NIB3);
    GET_SRC(OP1,NIB1);
    UINT32 idx = RL(dst);
    while (cnt-- >= 0) {
        WRMEMSEG_W( idx, RW(src) );
		idx = (idx + 2) & 0x7f00ffff;
		src = (src+1) & 15;
    }
}

/******************************************
 ldm     rd,@rs,n
 flags:  ------                                                     *
 ******************************************/
static void Z1C_ssN0_0001_0000_dddd_0000_nmin1_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_CNT(OP1,NIB3);
	GET_DST(OP1,NIB1);
	UINT32 idx = RL(src);
	while (cnt-- >= 0) {
		RW(dst) = RDMEMSEG_W( idx );
		idx = (idx + 2) & 0x3f00ffff;
		dst = (dst+1) & 15;
    }
}

/******************************************
 ldl     @rd,rrs
 flags:  ------                                                     *
 ******************************************/
static void Z1D_ddN0_ssss_segmented(void)
{
	GET_SRC(OP0,NIB3);
	GET_DST(OP0,NIB2);
	WRMEMSEG_L( RL(dst), RL(src) );
}

/******************************************
 jp      cc,rd
 flags:  ------                                                     *
 ******************************************/
static void Z1E_ddN0_cccc_segmented(void)
{
        UINT32 seg_addr;
	GET_CCC(OP0,NIB3);
	GET_DST(OP0,NIB2);
	switch (cc) {
		case  0: if (CC0) PC = RL(dst) & 0x7f00ffff; break;
		case  1: if (CC1) PC = RL(dst) & 0x7f00ffff; break;
		case  2: if (CC2) PC = RL(dst) & 0x7f00ffff; break;
		case  3: if (CC3) PC = RL(dst) & 0x7f00ffff; break;
		case  4: if (CC4) PC = RL(dst) & 0x7f00ffff; break;
		case  5: if (CC5) PC = RL(dst) & 0x7f00ffff; break;
		case  6: if (CC6) PC = RL(dst) & 0x7f00ffff; break;
		case  7: if (CC7) PC = RL(dst) & 0x7f00ffff; break;
		case  8: if (CC8) PC = RL(dst) & 0x7f00ffff; break;
		case  9: if (CC9) PC = RL(dst) & 0x7f00ffff; break;
		case 10: if (CCA) PC = RL(dst) & 0x7f00ffff; break;
		case 11: if (CCB) PC = RL(dst) & 0x7f00ffff; break;
		case 12: if (CCC) PC = RL(dst) & 0x7f00ffff; break;
		case 13: if (CCD) PC = RL(dst) & 0x7f00ffff; break;
		case 14: if (CCE) PC = RL(dst) & 0x7f00ffff; break;
		case 15: if (CCF) PC = RL(dst) & 0x7f00ffff; break;
	}
	CHANGE_PC_PA(seg_addr);
}

/******************************************
 call    @rd
 flags:  ------                                                     *
 ******************************************/
static void Z1F_ddN0_0000_segmented(void)
{
        UINT32 seg_addr;
	GET_DST(OP0,NIB2);
	PUSHLSEG( SPSEG, PC );
    	PC = RL(dst) & 0x7f00ffff;
	CHANGE_PC_PA(seg_addr);
}

/******************************************
 ldb     rbd,@rs
 flags:  ------                                                     *
 ******************************************/
static void Z20_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	RB(dst) = RDMEMSEG_B( RL(src) );
}

/******************************************
 ld      rd,@rs
 flags:  ------                                                     *
 ******************************************/
static void Z21_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	RW(dst) = RDMEMSEG_W( RL(src) );
}

/******************************************
 resb    @rd,imm4
 flags:  ------                                                     *
 ******************************************/
static void Z22_ddN0_imm4_segmented(void)
{
	GET_BIT(OP0);
	GET_DST(OP0,NIB2);
	WRMEMSEG_B(RL(dst), RDMEMSEG_B(RL(dst)) & ~bit);
}

/******************************************
 res     @rd,imm4
 flags:  ------                                                     *
 ******************************************/
static void Z23_ddN0_imm4_segmented(void)
{
	GET_BIT(OP0);
	GET_DST(OP0,NIB2);
	WRMEMSEG_W(RL(dst), RDMEMSEG_W(RL(dst)) & ~bit);
}

/******************************************
 setb    @rd,imm4
 flags:  ------                                                     *
 ******************************************/
static void Z24_ddN0_imm4_segmented(void)
{
	GET_BIT(OP0);
	GET_DST(OP0,NIB2);
	WRMEMSEG_B(RL(dst), RDMEMSEG_B(RL(dst)) | bit);
}

/******************************************
 set     @rd,imm4
 flags:  ------                                                     *
 ******************************************/
static void Z25_ddN0_imm4_segmented(void)
{
	GET_BIT(OP0);
	GET_DST(OP0,NIB2);
	WRMEMSEG_W(RL(dst), RDMEMSEG_W(RL(dst)) | bit);
}

/******************************************
 bitb    @rd,imm4
 flags:  -Z----                                                     *
 ******************************************/
static void Z26_ddN0_imm4_segmented(void)
{
	GET_BIT(OP0);
	GET_DST(OP0,NIB2);
	if (RDMEMSEG_B(RL(dst)) & bit) CLR_Z; else SET_Z;
}

/******************************************
 bit     @rd,imm4
 flags:  -Z----                                                     *
 ******************************************/
static void Z27_ddN0_imm4_segmented(void)
{
	GET_BIT(OP0);
	GET_DST(OP0,NIB2);
	if (RDMEMSEG_W(RL(dst)) & bit) CLR_Z; else SET_Z;
}

/******************************************
 incb    @rd,imm4m1
 flags:  -ZSV--                                                     *
 ******************************************/
static void Z28_ddN0_imm4m1_segmented(void)
{
	GET_I4M1(OP0,NIB3);
	GET_DST(OP0,NIB2);
	WRMEMSEG_B( RL(dst), INCB( RDMEMSEG_B(RL(dst)), i4p1) );
}

/******************************************
 inc     @rd,imm4m1
 flags:  -ZSV--                                                     *
 ******************************************/
static void Z29_ddN0_imm4m1_segmented(void)
{
	GET_I4M1(OP0,NIB3);
	GET_DST(OP0,NIB2);
	WRMEMSEG_W( RL(dst), INCW( RDMEMSEG_W(RL(dst)), i4p1 ) );
}

/******************************************
 decb    @rd,imm4m1
 flags:  -ZSV--                                                     *
 ******************************************/
static void Z2A_ddN0_imm4m1_segmented(void)
{
	GET_I4M1(OP0,NIB3);
	GET_DST(OP0,NIB2);
	WRMEMSEG_B( RL(dst), DECB( RDMEMSEG_B(RL(dst)), i4p1 ) );
}

/******************************************
 dec     @rd,imm4m1
 flags:  -ZSV--                                                     *
 ******************************************/
static void Z2B_ddN0_imm4m1_segmented(void)
{
	GET_I4M1(OP0,NIB3);
	GET_DST(OP0,NIB2);
	WRMEMSEG_W( RL(dst), DECW( RDMEMSEG_W(RL(dst)), i4p1 ) );
}

/******************************************
 exb     rbd,@rs
 flags:  ------                                                     *
 ******************************************/
static void Z2C_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	UINT8 tmp = RDMEMSEG_B( RL(src) );
	WRMEMSEG_B( RL(src), RB(dst) );
	RB(dst) = tmp;
}

/******************************************
 ex      rd,@rs
 flags:  ------                                                     *
 ******************************************/
static void Z2D_ssN0_dddd_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	UINT16 tmp = RDMEMSEG_W( RL(src) );
	WRMEMSEG_W( RL(src), RW(dst) );
	RW(dst) = tmp;
}

/******************************************
 ldb     @rd,rbs
 flags:  ------                                                     *
 ******************************************/
static void Z2E_ddN0_ssss_segmented(void)
{
	GET_SRC(OP0,NIB3);
	GET_DST(OP0,NIB2);
	WRMEMSEG_B( RL(dst), RB(src) );
}

/******************************************
 ld      @rd,rs
 flags:  ------                                                     *
 ******************************************/
static void Z2F_ddN0_ssss_segmented(void)
{
	GET_SRC(OP0,NIB3);
	GET_DST(OP0,NIB2);
	WRMEMSEG_W( RL(dst), RW(src) );
}

/******************************************
 ldb     rbd,rs(imm16)
 flags:  ------                                                     *
 ******************************************/
static void Z30_ssN0_dddd_imm16_segmented(void)
{
        UINT32 seg_addr;
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_IMM16(OP1);
        seg_addr = RL(src) + imm16;
	RB(dst) = RDMEMSEG_B( seg_addr );
}

/******************************************
 ld      rd,rs(imm16)
 flags:  ------                                                     *
 ******************************************/
static void Z31_ssN0_dddd_imm16_segmented(void)
{
        UINT32 seg_addr;
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_IMM16(OP1);
        seg_addr = RL(src) + imm16;
	RW(dst) = RDMEMSEG_W( seg_addr );
}

/******************************************
 ldb     rd(imm16),rbs
 flags:  ------                                                     *
 ******************************************/
static void Z32_ddN0_ssss_imm16_segmented(void)
{
        UINT32 seg_addr;
	GET_SRC(OP0,NIB3);
	GET_DST(OP0,NIB2);
	GET_IMM16(OP1);
        seg_addr = RL(dst) + imm16;
	WRMEMSEG_B( seg_addr, RB(src) );
}

/******************************************
 ld      rd(imm16),rs
 flags:  ------                                                     *
 ******************************************/
static void Z33_ddN0_ssss_imm16_segmented(void)
{
        UINT32 seg_addr;
	GET_SRC(OP0,NIB3);
	GET_DST(OP0,NIB2);
	GET_IMM16(OP1);
        seg_addr = RL(dst) + imm16;
	WRMEMSEG_W( seg_addr, RW(src) );
}

/******************************************
 ldar    prd,dsp16
 flags:  ------                                                     *
 ******************************************/
static void Z34_0000_dddd_dsp16_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_DSP16;
	RL(dst) = (PC & 0x7f000000) | dsp16;
}

/******************************************
 lda     prd,rs(imm16)
 flags:  ------                                                     *
 ******************************************/
static void Z34_ssN0_dddd_imm16_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_IMM16(OP1);
        RL(dst) = RL(src) + imm16;
}

/******************************************
 ldl     rrd,rs(imm16)
 flags:  ------                                                     *
 ******************************************/
static void Z35_ssN0_dddd_imm16_segmented(void)
{
        UINT32 seg_addr;
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_IMM16(OP1);
        seg_addr = RL(src) + imm16;
	RL(dst) = RDMEMSEG_L( seg_addr );
}

/******************************************
 ldl     rrd(imm16),rrs
 flags:  ------                                                     *
 ******************************************/
static void Z37_ddN0_ssss_imm16_segmented(void)
{
        UINT32 seg_addr;
        GET_SRC(OP0,NIB3);
        GET_DST(OP0,NIB2);
        GET_IMM16(OP1);
        seg_addr = RL(dst) + imm16;
        WRMEMSEG_L( seg_addr, RL(src) );
}

/******************************************
 ldps    @rs
 flags:  CZSVDH                                                     *
 ******************************************/
static void Z39_ssN0_0000_segmented(void)
{
        UINT32 seg_addr;
	GET_SRC(OP0,NIB2);
	UINT16 fcw;
        Z8001_PRIV_INST_CHECK;

	fcw = RDMEMSEG_W( RL(src) + 2 );
	PC  = RDMEMSEG_L( RL(src) + 4);
	CHANGE_FCW(fcw); /* check for user/system mode change */
	CHANGE_PC_PA(seg_addr);
}

/******************************************
 inib(r) @rd,@rs,ra
 flags:  ---V--                                                     *
 ******************************************/
static void Z3A_ssss_0000_0000_aaaa_dddd_x000_segmented(void)
{
    GET_SRC(OP0,NIB2);
    GET_CNT(OP1,NIB1);
    GET_DST(OP1,NIB2);
    GET_CCC(OP1,NIB3);
    Z8001_PRIV_INST_CHECK;

    WRMEMSEG_B( RL(dst), RDPORT_B( 0, RW(src) ) );
    RW(dst+1)++;
//    RW(src)++;                                                      // !!!! BUG
	if (--RW(cnt)) { CLR_V; if (cc == 0) PC -= 4; } else SET_V;
}

/******************************************
 sinib   @rd,@rs,ra
 sinibr  @rd,@rs,ra
 flags:  ------                                                     *
 ******************************************/
static void Z3A_ssss_0001_0000_aaaa_dddd_x000_segmented(void)
{
    GET_SRC(OP0,NIB2);
    GET_CNT(OP1,NIB1);
    GET_DST(OP1,NIB2);
    GET_CCC(OP1,NIB3);
    Z8001_PRIV_INST_CHECK;

    WRMEMSEG_B( RL(dst), RDPORT_B( 1, RW(src) ) );
    RW(dst+1)++;
//    RW(src)++;                                                      // !!!!! BUG
	if (--RW(cnt)) { CLR_V; if (cc == 0) PC -= 4; } else SET_V;
}

/******************************************
 outib   @rd,@rs,ra
 outibr  @rd,@rs,ra
 flags:  ---V--                                                     *
 ******************************************/
static void Z3A_ssss_0010_0000_aaaa_dddd_x000_segmented(void)
{
    GET_SRC(OP0,NIB2);
    GET_CNT(OP1,NIB1);
    GET_DST(OP1,NIB2);
    GET_CCC(OP1,NIB3);
    Z8001_PRIV_INST_CHECK;

    WRPORT_B( 0, RW(dst), RDMEMSEG_B( RL(src) ) );
//    RW(dst)++;                                                        // !!!! BUG
    RW(src+1)++;
	if (--RW(cnt)) { CLR_V; if (cc == 0) PC -= 4; } else SET_V;
}

/******************************************
 soutib  @rd,@rs,ra
 soutibr @rd,@rs,ra
 flags:  ------                                                     *
 ******************************************/
static void Z3A_ssss_0011_0000_aaaa_dddd_x000_segmented(void)
{
    GET_SRC(OP0,NIB2);
    GET_CNT(OP1,NIB1);
    GET_DST(OP1,NIB2);
    GET_CCC(OP1,NIB3);
    Z8001_PRIV_INST_CHECK;

    WRPORT_B( 1, RW(dst), RDMEMSEG_B( RL(src) ) );
//    RW(dst)++;                                                       // !!!!! BUG
    RW(src+1)++;
	if (--RW(cnt)) { CLR_V; if (cc == 0) PC -= 4; } else SET_V;
}

/******************************************
 indb    @rd,@rs,rba
 indbr   @rd,@rs,rba
 flags:  ---V--                                                     *
 ******************************************/
static void Z3A_ssss_1000_0000_aaaa_dddd_x000_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_CNT(OP1,NIB1);
	GET_DST(OP1,NIB2);
	GET_CCC(OP1,NIB3);
        Z8001_PRIV_INST_CHECK;

	WRMEMSEG_B( RL(dst), RDPORT_B( 0, RW(src) ) );
	RW(dst+1)--;
//	RW(src)--;                                                     // !!!!! BUG
	if (--RW(cnt)) { CLR_V; if (cc == 0) PC -= 4; } else SET_V;
}

/******************************************
 sindb   @rd,@rs,rba
 sindbr  @rd,@rs,rba
 flags:  ------                                                     *
 ******************************************/
static void Z3A_ssss_1001_0000_aaaa_dddd_x000_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_CNT(OP1,NIB1);
	GET_DST(OP1,NIB2);
	GET_CCC(OP1,NIB3);
        Z8001_PRIV_INST_CHECK;

	WRMEMSEG_B( RL(dst), RDPORT_B( 1, RW(src) ) );
	RW(dst+1)--;
//	RW(src)--;                                                   // !!!!! BUG
	if (--RW(cnt)) { CLR_V; if (cc == 0) PC -= 4; } else SET_V;
}

/******************************************
 outdb   @rd,@rs,rba
 outdbr  @rd,@rs,rba
 flags:  ---V--                                                     *
 ******************************************/
static void Z3A_ssss_1010_0000_aaaa_dddd_x000_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_CNT(OP1,NIB1);
	GET_DST(OP1,NIB2);
	GET_CCC(OP1,NIB3);
        Z8001_PRIV_INST_CHECK;

	WRPORT_B( 0, RW(dst), RDMEMSEG_B( RL(src) ) );
//	RW(dst)--;                                                     // !!!!!! BUG
	RW(src+1)--;
	if (--RW(cnt)) { CLR_V; if (cc == 0) PC -= 4; } else SET_V;
}

/******************************************
 soutdb  @rd,@rs,rba
 soutdbr @rd,@rs,rba
 flags:  ------                                                     *
 ******************************************/
static void Z3A_ssss_1011_0000_aaaa_dddd_x000_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_CNT(OP1,NIB1);
	GET_DST(OP1,NIB2);
	GET_CCC(OP1,NIB3);
        Z8001_PRIV_INST_CHECK;

	WRPORT_B( 1, RW(dst), RDMEMSEG_B( RL(src) ) );
//	RW(dst)--;                                                     // !!!!! BUG
	RW(src+1)--;
	if (--RW(cnt)) { CLR_V; if (cc == 0) PC -= 4; } else SET_V;
}

/******************************************
 ini     @rd,@rs,ra
 inir    @rd,@rs,ra
 flags:  ---V--                                                     *
 ******************************************/
static void Z3B_ssss_0000_0000_aaaa_dddd_x000_segmented(void)
{
	GET_SRC(OP0,NIB2);
        GET_CNT(OP1,NIB1);
	GET_DST(OP1,NIB2);
	GET_CCC(OP1,NIB3);
        Z8001_PRIV_INST_CHECK;

	WRMEMSEG_W( RL(dst), RDPORT_W( 0, RW(src) ) );
	RW(dst+1) += 2;
//	RW(src) += 2;                                                 // !!!! BUG
	if (--RW(cnt)) { CLR_V; if (cc == 0) PC -= 4; } else SET_V;
}

/******************************************
 sini    @rd,@rs,ra
 sinir   @rd,@rs,ra
 flags:  ------                                                     *
 ******************************************/
static void Z3B_ssss_0001_0000_aaaa_dddd_x000_segmented(void)
{
	GET_SRC(OP0,NIB2);
        GET_CNT(OP1,NIB1);
	GET_DST(OP1,NIB2);
	GET_CCC(OP1,NIB3);
        Z8001_PRIV_INST_CHECK;

	WRMEMSEG_W( RL(dst), RDPORT_W( 1, RW(src) ) );
	RW(dst+1) += 2;
//	RW(src) += 2;                                                 // !!!! BUG
	if (--RW(cnt)) { CLR_V; if (cc == 0) PC -= 4; } else SET_V;
}

/******************************************
 outi    @rd,@rs,ra
 outir   @rd,@rs,ra
 flags:  ---V--                                                     *
 ******************************************/
static void Z3B_ssss_0010_0000_aaaa_dddd_x000_segmented(void)
{
	GET_SRC(OP0,NIB2);
        GET_CNT(OP1,NIB1);
	GET_DST(OP1,NIB2);
	GET_CCC(OP1,NIB3);
        Z8001_PRIV_INST_CHECK;

	WRPORT_W( 0, RW(dst), RDMEMSEG_W( RL(src) ) );
//	RW(dst) += 2;                                                  // !!!!! BUG
	RW(src+1) += 2;
	if (--RW(cnt)) { CLR_V; if (cc == 0) PC -= 4; } else SET_V;
}

/******************************************
 souti   @rd,@rs,ra
 soutir  @rd,@rs,ra
 flags:  ------                                                     *
 ******************************************/
static void Z3B_ssss_0011_0000_aaaa_dddd_x000_segmented(void)
{
	GET_SRC(OP0,NIB2);
    	GET_CNT(OP1,NIB1);
	GET_DST(OP1,NIB2);
	GET_CCC(OP1,NIB3);
        Z8001_PRIV_INST_CHECK;

	WRPORT_W( 1, RW(dst), RDMEMSEG_W( RL(src) ) );
//	RW(dst) += 2;                                                 // !!!!! BUG
	RW(src+1) += 2;
	if (--RW(cnt)) { CLR_V; if (cc == 0) PC -= 4; } else SET_V;
}

/******************************************
 ind     @rd,@rs,ra
 indr    @rd,@rs,ra
 flags:  ---V--                                                     *
 ******************************************/
static void Z3B_ssss_1000_0000_aaaa_dddd_x000_segmented(void)
{
	GET_SRC(OP0,NIB2);
    	GET_CNT(OP1,NIB1);
	GET_DST(OP1,NIB2);
	GET_CCC(OP1,NIB3);
        Z8001_PRIV_INST_CHECK;

	WRMEMSEG_W( RL(dst), RDPORT_W( 0, RW(src) ) );
	RW(dst+1) -= 2;
//	RW(src) -= 2;                                                  // !!!!! BUG
	if (--RW(cnt)) { CLR_V; if (cc == 0) PC -= 4; } else SET_V;
}

/******************************************
 sind    @rd,@rs,ra
 sindr   @rd,@rs,ra
 flags:  ------                                                     *
 ******************************************/
static void Z3B_ssss_1001_0000_aaaa_dddd_x000_segmented(void)
{
	GET_SRC(OP0,NIB2);
    	GET_CNT(OP1,NIB1);
	GET_DST(OP1,NIB2);
	GET_CCC(OP1,NIB3);
        Z8001_PRIV_INST_CHECK;

	WRMEMSEG_W( RL(dst), RDPORT_W( 1, RW(src) ) );
	RW(dst+1) -= 2;
//	RW(src) -= 2;                                                 // !!!!! BUG
	if (--RW(cnt)) { CLR_V; if (cc == 0) PC -= 4; } else SET_V;
}

/******************************************
 outd    @rd,@rs,ra
 outdr   @rd,@rs,ra
 flags:  ---V--                                                     *
 ******************************************/
static void Z3B_ssss_1010_0000_aaaa_dddd_x000_segmented(void)
{
	GET_SRC(OP0,NIB2);
    	GET_CNT(OP1,NIB1);
	GET_DST(OP1,NIB2);
	GET_CCC(OP1,NIB3);
        Z8001_PRIV_INST_CHECK;

	WRPORT_W( 0, RW(dst), RDMEMSEG_W( RL(src) ) );
//	RW(dst) -= 2;                                                  // !!!! BUG
	RW(src+1) -= 2;
	if (--RW(cnt)) { CLR_V; if (cc == 0) PC -= 4; } else SET_V;
}

/******************************************
 soutd   @rd,@rs,ra
 soutdr  @rd,@rs,ra
 flags:  ------                                                     *
 ******************************************/
static void Z3B_ssss_1011_0000_aaaa_dddd_x000_segmented(void)
{
	GET_SRC(OP0,NIB2);
        GET_CNT(OP1,NIB1);
	GET_DST(OP1,NIB2);
	GET_CCC(OP1,NIB3);
        Z8001_PRIV_INST_CHECK;

	WRPORT_W( 1, RW(dst), RDMEMSEG_W( RL(src) ) );
//	RW(dst) -= 2;                                                   // !!!! BUG
	RW(src+1) -= 2;
	if (--RW(cnt)) { CLR_V; if (cc == 0) PC -= 4; } else SET_V;
}

/******************************************
 addb    rbd,addr
 flags:  CZSVDH                                                     *
 ******************************************/
static void Z40_0000_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	RB(dst) = ADDB( RB(dst), RDMEMSEG_B(seg_addr));
}

/******************************************
 addb    rbd,addr(rs)
 flags:  CZSVDH                                                     *
 ******************************************/
static void Z40_ssN0_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	RB(dst) = ADDB( RB(dst), RDMEMSEG_B(seg_addr) );
}

/******************************************
 add     rd,addr
 flags:  CZSV--                                                     *
 ******************************************/
static void Z41_0000_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	RW(dst) = ADDW( RW(dst), RDMEMSEG_W(seg_addr)); /* EHC */
}

/******************************************
 add     rd,addr(rs)
 flags:  CZSV--                                                     *
 ******************************************/
static void Z41_ssN0_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	RW(dst) = ADDW( RW(dst), RDMEMSEG_W(seg_addr) );	/* ASG */
}

/******************************************
 subb    rbd,addr
 flags:  CZSVDH                                                     *
 ******************************************/
static void Z42_0000_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	RB(dst) = SUBB( RB(dst), RDMEMSEG_B(seg_addr)); /* EHC */
}

/******************************************
 subb    rbd,addr(rs)
 flags:  CZSVDH                                                     *
 ******************************************/
static void Z42_ssN0_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	RB(dst) = SUBB( RB(dst), RDMEMSEG_B(seg_addr) );
}

/******************************************
 sub     rd,addr
 flags:  CZSV--                                                     *
 ******************************************/
static void Z43_0000_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	RW(dst) = SUBW( RW(dst), RDMEMSEG_W(seg_addr) );
}

/******************************************
 sub     rd,addr(rs)
 flags:  CZSV--                                                     *
 ******************************************/
static void Z43_ssN0_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	RW(dst) = SUBW( RW(dst), RDMEMSEG_W(seg_addr) );
}

/******************************************
 orb     rbd,addr
 flags:  CZSP--                                                     *
 ******************************************/
static void Z44_0000_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	RB(dst) = ORB( RB(dst), RDMEMSEG_B(seg_addr) );
}

/******************************************
 orb     rbd,addr(rs)
 flags:  CZSP--                                                     *
 ******************************************/
static void Z44_ssN0_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	RB(dst) = ORB( RB(dst), RDMEMSEG_B(seg_addr) );
}

/******************************************
 or      rd,addr
 flags:  CZS---                                                     *
 ******************************************/
static void Z45_0000_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	RW(dst) = ORW( RW(dst), RDMEMSEG_W(seg_addr) );
}

/******************************************
 or      rd,addr(rs)
 flags:  CZS---                                                     *
 ******************************************/
static void Z45_ssN0_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	RW(dst) = ORW( RW(dst), RDMEMSEG_W(seg_addr) );
}

/******************************************
 andb    rbd,addr
 flags:  -ZSP--                                                     *
 ******************************************/
static void Z46_0000_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	RB(dst) = ANDB( RB(dst), RDMEMSEG_B(seg_addr) );
}

/******************************************
 andb    rbd,addr(rs)
 flags:  -ZSP--                                                     *
 ******************************************/
static void Z46_ssN0_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	RB(dst) = ANDB( RB(dst), RDMEMSEG_B(seg_addr) );
}

/******************************************
 and     rd,addr
 flags:  -ZS---                                                     *
 ******************************************/
static void Z47_0000_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	RW(dst) = ANDW( RW(dst), RDMEMSEG_W(seg_addr) );
}

/******************************************
 and     rd,addr(rs)
 flags:  -ZS---                                                     *
 ******************************************/
static void Z47_ssN0_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	RW(dst) = ANDW( RW(dst), RDMEMSEG_W(seg_addr) );
}

/******************************************
 xorb    rbd,addr
 flags:  -ZSP--                                                     *
 ******************************************/
static void Z48_0000_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	RB(dst) = XORB( RB(dst), RDMEMSEG_B(seg_addr) );
}

/******************************************
 xorb    rbd,addr(rs)
 flags:  -ZSP--                                                     *
 ******************************************/
static void Z48_ssN0_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	RB(dst) = XORB( RB(dst), RDMEMSEG_B(seg_addr) );
}

/******************************************
 xor     rd,addr
 flags:  -ZS---                                                     *
 ******************************************/
static void Z49_0000_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	RW(dst) = XORW( RW(dst), RDMEMSEG_W(seg_addr) );
}

/******************************************
 xor     rd,addr(rs)
 flags:  -ZS---                                                     *
 ******************************************/
static void Z49_ssN0_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	RW(dst) = XORW( RW(dst), RDMEMSEG_W(seg_addr) );
}

/******************************************
 cpb     rbd,addr
 flags:  CZSV--                                                     *
 ******************************************/
static void Z4A_0000_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	CPB( RB(dst), RDMEMSEG_B(seg_addr) );
}

/******************************************
 cpb     rbd,addr(rs)
 flags:  CZSV--                                                     *
 ******************************************/
static void Z4A_ssN0_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	CPB( RB(dst), RDMEMSEG_B(seg_addr) );
}

/******************************************
 cp      rd,addr
 flags:  CZSV--                                                     *
 ******************************************/
static void Z4B_0000_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	CPW( RW(dst), RDMEMSEG_W(seg_addr) );
}

/******************************************
 cp      rd,addr(rs)
 flags:  CZSV--                                                     *
 ******************************************/
static void Z4B_ssN0_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	CPW( RW(dst), RDMEMSEG_W(seg_addr) );
}

/******************************************
 comb    addr
 flags:  -ZSP--                                                     *
 ******************************************/
static void Z4C_0000_0000_addr_segmented(void)
{
	GET_SEGADDR(OP1);
	WRMEMSEG_B( seg_addr, COMB(RDMEMSEG_W(seg_addr)) );
}

/******************************************
 cpb     addr,imm8
 flags:  CZSV--                                                     *
 ******************************************/
static void Z4C_0000_0001_addr_imm8_segmented(void)
{
	GET_SEGADDR_IMM8(OP1);
	CPB( RDMEMSEG_B(seg_addr), imm8 );
}

/******************************************
 negb    addr
 flags:  CZSV--                                                     *
 ******************************************/
static void Z4C_0000_0010_addr_segmented(void)
{
	GET_SEGADDR(OP1);
	WRMEMSEG_B( seg_addr, NEGB(RDMEMSEG_B(seg_addr)) );
}

/******************************************
 testb   addr
 flags:  -ZSP--                                                     *
 ******************************************/
static void Z4C_0000_0100_addr_segmented(void)
{
	GET_SEGADDR(OP1);
	TESTB(RDMEMSEG_B(seg_addr));
}

/******************************************
 ldb     addr,imm8
 flags:  ------                                                     *
 ******************************************/
static void Z4C_0000_0101_addr_imm8_segmented(void)
{
	GET_SEGADDR_IMM8(OP1);
	WRMEMSEG_B( seg_addr, imm8 );
}

/******************************************
 tsetb   addr
 flags:  --S---                                                     *
 ******************************************/
static void Z4C_0000_0110_addr_segmented(void)
{
	GET_SEGADDR(OP1);
    if (RDMEMSEG_B(seg_addr) & S08) SET_S; else CLR_S;
    WRMEMSEG_B(seg_addr, 0xff);
}

/******************************************
 clrb    addr
 flags:  ------                                                     *
 ******************************************/
static void Z4C_0000_1000_addr_segmented(void)
{
	GET_SEGADDR(OP1);
	WRMEMSEG_B( seg_addr, 0 );
}

/******************************************
 comb    addr(rd)
 flags:  -ZSP--                                                     *
 ******************************************/
static void Z4C_ddN0_0000_addr_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	WRMEMSEG_B( seg_addr, COMB(RDMEMSEG_B(seg_addr)) );
}

/******************************************
 cpb     addr(rd),imm8
 flags:  CZSV--                                                     *
 ******************************************/
static void Z4C_ddN0_0001_addr_imm8_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SEGADDR_IMM8(OP1);
	seg_addr += RW(dst);
	CPB( RDMEMSEG_B(seg_addr), imm8 );
}

/******************************************
 negb    addr(rd)
 flags:  CZSV--                                                     *
 ******************************************/
static void Z4C_ddN0_0010_addr_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	WRMEMSEG_B( seg_addr, NEGB(RDMEMSEG_B(seg_addr)) );
}

/******************************************
 testb   addr(rd)
 flags:  -ZSP--                                                     *
 ******************************************/
static void Z4C_ddN0_0100_addr_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	TESTB( RDMEMSEG_B(seg_addr) );
}

/******************************************
 ldb     addr(rd),imm8
 flags:  ------                                                     *
 ******************************************/
static void Z4C_ddN0_0101_addr_imm8_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SEGADDR_IMM8(OP1);
	seg_addr += RW(dst);
	WRMEMSEG_B( seg_addr, imm8 );
}

/******************************************
 tsetb   addr(rd)
 flags:  --S---                                                     *
 ******************************************/
static void Z4C_ddN0_0110_addr_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
    if (RDMEMSEG_B(seg_addr) & S08) SET_S; else CLR_S;
    WRMEMSEG_B(seg_addr, 0xff);
}

/******************************************
 clrb    addr(rd)
 flags:  ------                                                     *
 ******************************************/
static void Z4C_ddN0_1000_addr_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	WRMEMSEG_B( seg_addr, 0 );
}

/******************************************
 com     addr
 flags:  -ZS---                                                     *
 ******************************************/
static void Z4D_0000_0000_addr_segmented(void)
{
	GET_SEGADDR(OP1);
	WRMEMSEG_W( seg_addr, COMW(RDMEMSEG_W(seg_addr)) );
}

/******************************************
 cp      addr,imm16
 flags:  CZSV--                                                     *
 ******************************************/
static void Z4D_0000_0001_addr_imm16_segmented(void)
{
	GET_SEGADDR_IMM16(OP1);
	CPW( RDMEMSEG_W(seg_addr), imm16 );
}

/******************************************
 neg     addr
 flags:  CZSV--                                                     *
 ******************************************/
static void Z4D_0000_0010_addr_segmented(void)
{
	GET_SEGADDR(OP1);
	WRMEMSEG_W( seg_addr, NEGW(RDMEMSEG_W(seg_addr)) );
}

/******************************************
 test    addr
 flags:  ------                                                     *
 ******************************************/
static void Z4D_0000_0100_addr_segmented(void)
{
	GET_SEGADDR(OP1);
	TESTW( RDMEMSEG_W(seg_addr) );
}

/******************************************
 ld      addr,imm16
 flags:  ------                                                     *
 ******************************************/
static void Z4D_0000_0101_addr_imm16_segmented(void)
{
	GET_SEGADDR_IMM16(OP1);
	WRMEMSEG_W( seg_addr, imm16 );
}

/******************************************
 tset    addr
 flags:  --S---                                                     *
 ******************************************/
static void Z4D_0000_0110_addr_segmented(void)
{
    GET_SEGADDR(OP1);
    if (RDMEMSEG_W(seg_addr) & S16) SET_S; else CLR_S;
    WRMEMSEG_W(seg_addr, 0xffff);
}

/******************************************
 clr     addr
 flags:  ------                                                     *
 ******************************************/
static void Z4D_0000_1000_addr_segmented(void)
{
	GET_SEGADDR(OP1);
	WRMEMSEG_W( seg_addr, 0 );
}

/******************************************
 com     addr(rd)
 flags:  -ZS---                                                     *
 ******************************************/
static void Z4D_ddN0_0000_addr_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	WRMEMSEG_W( seg_addr, COMW(RDMEMSEG_W(seg_addr)) );
}

/******************************************
 cp      addr(rd),imm16
 flags:  CZSV--                                                     *
 ******************************************/
static void Z4D_ddN0_0001_addr_imm16_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SEGADDR_IMM16(OP1);
	seg_addr += RW(dst);
	CPW( RDMEMSEG_W(seg_addr), imm16 );
}

/******************************************
 neg     addr(rd)
 flags:  CZSV--                                                     *
 ******************************************/
static void Z4D_ddN0_0010_addr_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	WRMEMSEG_W( seg_addr, NEGW(RDMEMSEG_W(seg_addr)) );
}

/******************************************
 test    addr(rd)
 flags:  ------                                                     *
 ******************************************/
static void Z4D_ddN0_0100_addr_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	TESTW( RDMEMSEG_W(seg_addr) );
}

/******************************************
 ld      addr(rd),imm16
 flags:  ------                                                     *
 ******************************************/
static void Z4D_ddN0_0101_addr_imm16_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SEGADDR_IMM16(OP1);
	seg_addr += RW(dst);
	WRMEMSEG_W( seg_addr, imm16 );
}

/******************************************
 tset    addr(rd)
 flags:  --S---                                                     *
 ******************************************/
static void Z4D_ddN0_0110_addr_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
    if (RDMEMSEG_W(seg_addr) & S16) SET_S; else CLR_S;
    WRMEMSEG_W(seg_addr, 0xffff);
}

/******************************************
 clr     addr(rd)
 flags:  ------                                                     *
 ******************************************/
static void Z4D_ddN0_1000_addr_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	WRMEMSEG_W( seg_addr, 0 );
}

/******************************************
 ldb     addr(rd),rbs
 flags:  ------                                                     *
 ******************************************/
static void Z4E_ddN0_ssN0_addr_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SRC(OP0,NIB3);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	WRMEMSEG_B( seg_addr, RB(src) );
}

/******************************************
 cpl     rrd,addr
 flags:  CZSV--                                                     *
 ******************************************/
static void Z50_0000_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	CPL( RL(dst), RDMEMSEG_L(seg_addr) );
}

/******************************************
 cpl     rrd,addr(rs)
 flags:  CZSV--                                                     *
 ******************************************/
static void Z50_ssN0_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	CPL( RL(dst), RDMEMSEG_L(seg_addr) );
}

/******************************************
 pushl   @rd,addr
 flags:  ------                                                     *
 ******************************************/
static void Z51_ddN0_0000_addr_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	PUSHLSEG( dst, RDMEMSEG_L(seg_addr) );
}

/******************************************
 pushl   @rd,addr(rs)
 flags:  ------                                                     *
 ******************************************/
static void Z51_ddN0_ssN0_addr_segmented(void)
{
	GET_SRC(OP0,NIB3);
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	PUSHLSEG( dst, RDMEMSEG_L(seg_addr) );
}

/******************************************
 subl    rrd,addr
 flags:  CZSV--                                                     *
 ******************************************/
static void Z52_0000_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	RL(dst) = SUBL( RL(dst), RDMEMSEG_L(seg_addr) );
}

/******************************************
 subl    rrd,addr(rs)
 flags:  CZSV--                                                     *
 ******************************************/
static void Z52_ssN0_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	RL(dst) = SUBL( RL(dst), RDMEMSEG_L(seg_addr) );
}

/******************************************
 push    @rd,addr
 flags:  ------                                                     *
 ******************************************/
static void Z53_ddN0_0000_addr_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	PUSHWSEG( dst, RDMEMSEG_W(seg_addr) );
}

/******************************************
 push    @rd,addr(rs)
 flags:  ------                                                     *
 ******************************************/
static void Z53_ddN0_ssN0_addr_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SRC(OP0,NIB3);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	PUSHWSEG( dst, RDMEMSEG_W(seg_addr) );
}

/******************************************
 ldl     rrd,addr
 flags:  ------                                                     *
 ******************************************/
static void Z54_0000_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	RL(dst) = RDMEMSEG_L( seg_addr );
}

/******************************************
 ldl     rrd,addr(rs)
 flags:  ------                                                     *
 ******************************************/
static void Z54_ssN0_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	RL(dst) = RDMEMSEG_L( seg_addr );
}

/******************************************
 popl    addr,@rs
 flags:  ------                                                     *
 ******************************************/
static void Z55_ssN0_0000_addr_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	WRMEMSEG_L( seg_addr, POPLSEG(src) );
}

/******************************************
 popl    addr(rd),@rs
 flags:  ------                                                     *
 ******************************************/
static void Z55_ssN0_ddN0_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	WRMEMSEG_L( seg_addr, POPLSEG(src) );
}

/******************************************
 addl    rrd,addr
 flags:  CZSV--                                                     *
 ******************************************/
static void Z56_0000_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	RL(dst) = ADDL( RL(dst), RDMEMSEG_L(seg_addr) );
}

/******************************************
 addl    rrd,addr(rs)
 flags:  CZSV--                                                     *
 ******************************************/
static void Z56_ssN0_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	RL(dst) = ADDL( RL(dst), RDMEMSEG_L(seg_addr) );
}

/******************************************
 pop     addr,@rs
 flags:  ------                                                     *
 ******************************************/
static void Z57_ssN0_0000_addr_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	WRMEMSEG_W( seg_addr, POPWSEG(src) );
}

/******************************************
 pop     addr(rd),@rs
 flags:  ------                                                     *
 ******************************************/
static void Z57_ssN0_ddN0_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	WRMEMSEG_W( seg_addr, POPWSEG(src) );
}

/******************************************
 multl   rqd,addr
 flags:  CZSV--                                                     *
 ******************************************/
static void Z58_0000_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	RQ(dst) = MULTL( RQ(dst), RDMEMSEG_L(seg_addr) );
}

/******************************************
 multl   rqd,addr(rs)
 flags:  CZSV--                                                     *
 ******************************************/
static void Z58_ssN0_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	RQ(dst) = MULTL( RQ(dst), RDMEMSEG_L(seg_addr) );
}

/******************************************
 mult    rrd,addr
 flags:  CZSV--                                                     *
 ******************************************/
static void Z59_0000_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	RL(dst) = MULTW( RL(dst), RDMEMSEG_W(seg_addr) );
}

/******************************************
 mult    rrd,addr(rs)
 flags:  CZSV--                                                     *
 ******************************************/
static void Z59_ssN0_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	RL(dst) = MULTW( RL(dst), RDMEMSEG_W(seg_addr) );
}

/******************************************
 divl    rqd,addr
 flags:  CZSV--                                                     *
 ******************************************/
static void Z5A_0000_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	RQ(dst) = DIVL( RQ(dst), RDMEMSEG_L(seg_addr) );
}

/******************************************
 divl    rqd,addr(rs)
 flags:  CZSV--                                                     *
 ******************************************/
static void Z5A_ssN0_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	RQ(dst) = DIVL( RQ(dst), RDMEMSEG_L(seg_addr) );
}

/******************************************
 div     rrd,addr
 flags:  CZSV--                                                     *
 ******************************************/
static void Z5B_0000_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	RL(dst) = DIVW( RL(dst), RDMEMSEG_W(seg_addr) );
}

/******************************************
 div     rrd,addr(rs)
 flags:  CZSV--                                                     *
 ******************************************/
static void Z5B_ssN0_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	RL(dst) = DIVW( RL(dst), RDMEMSEG_W(seg_addr) );
}

/******************************************
 ldm     rd,addr,n
 flags:  ------                                                     *
 ******************************************/
static void Z5C_0000_0001_0000_dddd_0000_nmin1_addr_segmented(void)
{
	GET_DST(OP1,NIB1);
	GET_CNT(OP1,NIB3);
	GET_SEGADDR(OP2);
	while (cnt-- >= 0) {
		RW(dst) = RDMEMSEG_W(seg_addr);
		dst = (dst+1) & 15;
		seg_addr = (seg_addr + 2) & 0x7f00ffff;
	}
}

/******************************************
 testl   addr
 flags:  -ZS---                                                     *
 ******************************************/
static void Z5C_0000_1000_addr_segmented(void)
{
	GET_SEGADDR(OP1);
	TESTL( RDMEMSEG_L(seg_addr) );
}

/******************************************
 ldm     addr,rs,n
 flags:  ------                                                     *
 ******************************************/
static void Z5C_0000_1001_0000_ssss_0000_nmin1_addr_segmented(void)
{
	GET_SRC(OP1,NIB1);
	GET_CNT(OP1,NIB3);
	GET_SEGADDR(OP2);
	while (cnt-- >= 0) {
		WRMEMSEG_W( seg_addr, RW(src) );
		src = (src+1) & 15;
		seg_addr = (seg_addr + 2) & 0x7f00ffff; 
	}
}

/******************************************
 testl   addr(rd)
 flags:  -ZS---                                                     *
 ******************************************/
static void Z5C_ddN0_1000_addr_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	TESTL( RDMEMSEG_L(seg_addr) );
}

/******************************************
 ldm     addr(rd),rs,n
 flags:  ------                                                     *
 ******************************************/
static void Z5C_ddN0_1001_0000_ssN0_0000_nmin1_addr_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SRC(OP1,NIB1);
	GET_CNT(OP1,NIB3);
	GET_SEGADDR(OP2);
	seg_addr += RW(dst);
	while (cnt-- >= 0) {
		WRMEMSEG_W( seg_addr, RW(src) );
		src = (src+1) & 15;
		seg_addr = (seg_addr + 2) & 0x7f00ffff; 
	}
}

/******************************************
 ldm     rd,addr(rs),n
 flags:  ------                                                     *
 ******************************************/
static void Z5C_ssN0_0001_0000_dddd_0000_nmin1_addr_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_DST(OP1,NIB1);
	GET_CNT(OP1,NIB3);
	GET_SEGADDR(OP2);
	seg_addr += RW(src);
	while (cnt-- >= 0) {
		RW(dst) = RDMEMSEG_W(seg_addr);
		dst = (dst+1) & 15;
		seg_addr = (seg_addr + 2) & 0x7f00ffff; 
	}
}

/******************************************
 ldl     addr,rrs
 flags:  ------                                                     *
 ******************************************/
static void Z5D_0000_ssss_addr_segmented(void)
{
	GET_SRC(OP0,NIB3);
	GET_SEGADDR(OP1);
	WRMEMSEG_L( seg_addr, RL(src) );
}

/******************************************
 ldl     addr(rd),rrs
 flags:  ------                                                     *
 ******************************************/
static void Z5D_ddN0_ssss_addr_segmented(void)
{
	GET_SRC(OP0,NIB3);
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	WRMEMSEG_L( seg_addr, RL(src) );
}

/******************************************
 jp      cc,addr
 flags:  ------                                                     *
 ******************************************/
static void Z5E_0000_cccc_addr_segmented(void)
{
	GET_CCC(OP0,NIB3);
	GET_SEGADDR(OP1);
	switch (cc) {
		case  0: if (CC0) PC = seg_addr; break;
		case  1: if (CC1) PC = seg_addr; break;
		case  2: if (CC2) PC = seg_addr; break;
		case  3: if (CC3) PC = seg_addr; break;
		case  4: if (CC4) PC = seg_addr; break;
		case  5: if (CC5) PC = seg_addr; break;
		case  6: if (CC6) PC = seg_addr; break;
		case  7: if (CC7) PC = seg_addr; break;
		case  8: if (CC8) PC = seg_addr; break;
		case  9: if (CC9) PC = seg_addr; break;
		case 10: if (CCA) PC = seg_addr; break;
		case 11: if (CCB) PC = seg_addr; break;
		case 12: if (CCC) PC = seg_addr; break;
		case 13: if (CCD) PC = seg_addr; break;
		case 14: if (CCE) PC = seg_addr; break;
		case 15: if (CCF) PC = seg_addr; break;
	}
	CHANGE_PC_PA(seg_addr);
}

/******************************************
 jp      cc,addr(rd)
 flags:  ------                                                     *
 ******************************************/
static void Z5E_ddN0_cccc_addr_segmented(void)
{
	GET_CCC(OP0,NIB3);
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	switch (cc) {
		case  0: if (CC0) PC = seg_addr; break;
		case  1: if (CC1) PC = seg_addr; break;
		case  2: if (CC2) PC = seg_addr; break;
		case  3: if (CC3) PC = seg_addr; break;
		case  4: if (CC4) PC = seg_addr; break;
		case  5: if (CC5) PC = seg_addr; break;
		case  6: if (CC6) PC = seg_addr; break;
		case  7: if (CC7) PC = seg_addr; break;
		case  8: if (CC8) PC = seg_addr; break;
		case  9: if (CC9) PC = seg_addr; break;
		case 10: if (CCA) PC = seg_addr; break;
		case 11: if (CCB) PC = seg_addr; break;
		case 12: if (CCC) PC = seg_addr; break;
		case 13: if (CCD) PC = seg_addr; break;
		case 14: if (CCE) PC = seg_addr; break;
		case 15: if (CCF) PC = seg_addr; break;
	}
	CHANGE_PC_PA(seg_addr);
}

/******************************************
 call    addr
 flags:  ------                                                     *
 ******************************************/
static void Z5F_0000_0000_addr_segmented(void)
{
	GET_SEGADDR(OP1);
	PUSHLSEG( SPSEG, PC );
	PC = seg_addr;
	CHANGE_PC_PA(seg_addr);
}

/******************************************
 call    addr(rd)
 flags:  ------                                                     *
 ******************************************/
static void Z5F_ddN0_0000_addr_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	PUSHLSEG( SPSEG, PC );
	seg_addr += RW(dst);
	PC = seg_addr;
	CHANGE_PC_PA(seg_addr);
}

/******************************************
 ldb     rbd,addr
 flags:  ------                                                     *
 ******************************************/
static void Z60_0000_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	RB(dst) = RDMEMSEG_B(seg_addr);
}

/******************************************
 ldb     rbd,addr(rs)
 flags:  ------                                                     *
 ******************************************/
static void Z60_ssN0_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	RB(dst) = RDMEMSEG_B(seg_addr);
}

/******************************************
 ld      rd,addr
 flags:  ------                                                     *
 ******************************************/
static void Z61_0000_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	RW(dst) = RDMEMSEG_W(seg_addr);
}

/******************************************
 ld      rd,addr(rs)
 flags:  ------                                                     *
 ******************************************/
static void Z61_ssN0_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	RW(dst) = RDMEMSEG_W(seg_addr);
}

/******************************************
 resb    addr,imm4
 flags:  ------                                                     *
 ******************************************/
static void Z62_0000_imm4_addr_segmented(void)
{
	GET_BIT(OP0);
	GET_SEGADDR(OP1);
	WRMEMSEG_B( seg_addr, RDMEMSEG_B(seg_addr) & ~bit );
}

/******************************************
 resb    addr(rd),imm4
 flags:  ------                                                     *
 ******************************************/
static void Z62_ddN0_imm4_addr_segmented(void)
{
	GET_BIT(OP0);
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	WRMEMSEG_B( seg_addr, RDMEMSEG_B(seg_addr) & ~bit );
}

/******************************************
 res     addr,imm4
 flags:  ------                                                     *
 ******************************************/
static void Z63_0000_imm4_addr_segmented(void)
{
	GET_BIT(OP0);
	GET_SEGADDR(OP1);
	WRMEMSEG_W( seg_addr, RDMEMSEG_W(seg_addr) & ~bit );
}

/******************************************
 res     addr(rd),imm4
 flags:  ------                                                     *
 ******************************************/
static void Z63_ddN0_imm4_addr_segmented(void)
{
	GET_BIT(OP0);
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	WRMEMSEG_W( seg_addr, RDMEMSEG_W(seg_addr) & ~bit );
}

/******************************************
 setb    addr,imm4
 flags:  ------                                                     *
 ******************************************/
static void Z64_0000_imm4_addr_segmented(void)
{
	GET_BIT(OP0);
	GET_SEGADDR(OP1);
	WRMEMSEG_B( seg_addr, RDMEMSEG_B(seg_addr) | bit );
}

/******************************************
 setb    addr(rd),imm4
 flags:  ------                                                     *
 ******************************************/
static void Z64_ddN0_imm4_addr_segmented(void)
{
	GET_BIT(OP0);
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	WRMEMSEG_B( seg_addr, RDMEMSEG_B(seg_addr) | bit );
}

/******************************************
 set     addr,imm4
 flags:  ------                                                     *
 ******************************************/
static void Z65_0000_imm4_addr_segmented(void)
{
	GET_BIT(OP0);
	GET_SEGADDR(OP1);
	WRMEMSEG_W( seg_addr, RDMEMSEG_W(seg_addr) | bit );
}

/******************************************
 set     addr(rd),imm4
 flags:  ------                                                     *
 ******************************************/
static void Z65_ddN0_imm4_addr_segmented(void)
{
	GET_BIT(OP0);
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	WRMEMSEG_W( seg_addr, RDMEMSEG_W(seg_addr) | bit );
}

/******************************************
 bitb    addr,imm4
 flags:  -Z----                                                     *
 ******************************************/
static void Z66_0000_imm4_addr_segmented(void)
{
	GET_BIT(OP0);
	GET_SEGADDR(OP1);
	if ( RDMEMSEG_B(seg_addr) & bit) CLR_Z; else SET_Z;
}

/******************************************
 bitb    addr(rd),imm4
 flags:  -Z----                                                     *
 ******************************************/
static void Z66_ddN0_imm4_addr_segmented(void)
{
	GET_BIT(OP0);
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
    	seg_addr += RW(dst);
    	if ( RDMEMSEG_B(seg_addr) & bit) CLR_Z; else SET_Z;
}

/******************************************
 bit     addr,imm4
 flags:  -Z----                                                     *
 ******************************************/
static void Z67_0000_imm4_addr_segmented(void)
{
	GET_BIT(OP0);
	GET_SEGADDR(OP1);
	if ( RDMEMSEG_W(seg_addr) & bit) CLR_Z; else SET_Z;
}

/******************************************
 bit     addr(rd),imm4
 flags:  -Z----                                                     *
 ******************************************/
static void Z67_ddN0_imm4_addr_segmented(void)
{
	GET_BIT(OP0);
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
    	seg_addr += RW(dst);
	if ( RDMEMSEG_W(seg_addr) & bit) CLR_Z; else SET_Z;
}

/******************************************
 incb    addr,imm4m1
 flags:  -ZSV--                                                     *
 ******************************************/
static void Z68_0000_imm4m1_addr_segmented(void)
{
	GET_I4M1(OP0,NIB3);
	GET_SEGADDR(OP1);
	WRMEMSEG_B( seg_addr, INCB(RDMEMSEG_B(seg_addr), i4p1) );
}

/******************************************
 incb    addr(rd),imm4m1
 flags:  -ZSV--                                                     *
 ******************************************/
static void Z68_ddN0_imm4m1_addr_segmented(void)
{
	GET_I4M1(OP0,NIB3);
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	WRMEMSEG_B( seg_addr, INCB(RDMEMSEG_B(seg_addr), i4p1) );
}

/******************************************
 inc     addr,imm4m1
 flags:  -ZSV--                                                     *
 ******************************************/
static void Z69_0000_imm4m1_addr_segmented(void)
{
	GET_I4M1(OP0,NIB3);
	GET_SEGADDR(OP1);
	WRMEMSEG_W( seg_addr, INCW(RDMEMSEG_W(seg_addr), i4p1) );
}

/******************************************
 inc     addr(rd),imm4m1
 flags:  -ZSV--                                                     *
 ******************************************/
static void Z69_ddN0_imm4m1_addr_segmented(void)
{
	GET_I4M1(OP0,NIB3);
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	WRMEMSEG_W( seg_addr, INCW(RDMEMSEG_W(seg_addr), i4p1) );
}

/******************************************
 decb    addr,imm4m1
 flags:  -ZSV--                                                     *
 ******************************************/
static void Z6A_0000_imm4m1_addr_segmented(void)
{
	GET_I4M1(OP0,NIB3);
	GET_SEGADDR(OP1);
	WRMEMSEG_B( seg_addr, DECB(RDMEMSEG_B(seg_addr), i4p1) );
}

/******************************************
 decb    addr(rd),imm4m1
 flags:  -ZSV--                                                     *
 ******************************************/
static void Z6A_ddN0_imm4m1_addr_segmented(void)
{
	GET_I4M1(OP0,NIB3);
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	WRMEMSEG_B( seg_addr, DECB(RDMEMSEG_B(seg_addr), i4p1) );
}

/******************************************
 dec     addr,imm4m1
 flags:  -ZSV--                                                     *
 ******************************************/
static void Z6B_0000_imm4m1_addr_segmented(void)
{
	GET_I4M1(OP0,NIB3);
	GET_SEGADDR(OP1);
	WRMEMSEG_W( seg_addr, DECW(RDMEMSEG_W(seg_addr), i4p1) );
}

/******************************************
 dec     addr(rd),imm4m1
 flags:  -ZSV--                                                     *
 ******************************************/
static void Z6B_ddN0_imm4m1_addr_segmented(void)
{
	GET_I4M1(OP0,NIB3);
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	WRMEMSEG_W( seg_addr, DECW(RDMEMSEG_W(seg_addr), i4p1) );
}

/******************************************
 exb     rbd,addr
 flags:  ------                                                     *
 ******************************************/
static void Z6C_0000_dddd_addr_segmented(void)
{
        UINT8 tmp;
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	tmp = RDMEMSEG_B(seg_addr);
	WRMEMSEG_B(seg_addr, RB(dst));
	RB(dst) = tmp;
}

/******************************************
 exb     rbd,addr(rs)
 flags:  ------                                                     *
 ******************************************/
static void Z6C_ssN0_dddd_addr_segmented(void)
{
	UINT8 tmp;
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	tmp = RDMEMSEG_B(seg_addr);
	WRMEMSEG_B(seg_addr, RB(dst));
    	RB(dst) = tmp;
}

/******************************************
 ex      rd,addr
 flags:  ------                                                     *
 ******************************************/
static void Z6D_0000_dddd_addr_segmented(void)
{
        UINT16 tmp;
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	tmp = RDMEMSEG_W(seg_addr);
	WRMEMSEG_W( seg_addr, RW(dst) );
	RW(dst) = tmp;
}

/******************************************
 ex      rd,addr(rs)
 flags:  ------                                                     *
 ******************************************/
static void Z6D_ssN0_dddd_addr_segmented(void)
{
	UINT16 tmp;
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
	tmp = RDMEMSEG_W(seg_addr);
	WRMEMSEG_W( seg_addr, RW(dst) );
    	RW(dst) = tmp;
}

/******************************************
 ldb     addr,rbs
 flags:  ------                                                     *
 ******************************************/
static void Z6E_0000_ssss_addr_segmented(void)
{
	GET_SRC(OP0,NIB3);
	GET_SEGADDR(OP1);
	WRMEMSEG_B( seg_addr, RB(src) );
}

/******************************************
 ldb     addr(rd),rbs
 flags:  ------                                                     *
 ******************************************/
static void Z6E_ddN0_ssss_addr_segmented(void)
{
	GET_SRC(OP0,NIB3);
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	WRMEMSEG_B( seg_addr, RB(src) );
}

/******************************************
 ld      addr,rs
 flags:  ------                                                     *
 ******************************************/
static void Z6F_0000_ssss_addr_segmented(void)
{
	GET_SRC(OP0,NIB3);
	GET_SEGADDR(OP1);
	WRMEMSEG_W( seg_addr, RW(src) );
}

/******************************************
 ld      addr(rd),rs
 flags:  ------                                                     *
 ******************************************/
static void Z6F_ddN0_ssss_addr_segmented(void)
{
	GET_SRC(OP0,NIB3);
	GET_DST(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(dst);
	WRMEMSEG_W( seg_addr, RW(src) );
}

/******************************************
 ldb     rbd,rs(rx)
 flags:  ------                                                     *
 ******************************************/
static void Z70_ssN0_dddd_0000_xxxx_0000_0000_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_IDX(OP1,NIB1);
	RB(dst) = RDMEMSEG_B((RL(src) + RW(idx)) );
}

/******************************************
 ld      rd,rs(rx)
 flags:  ------                                                     *
 ******************************************/
static void Z71_ssN0_dddd_0000_xxxx_0000_0000_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_IDX(OP1,NIB1);
	RW(dst) = RDMEMSEG_W( (RL(src) + RW(idx)) );
}

/******************************************
 ldb     rd(rx),rbs
 flags:  ------                                                     *
 ******************************************/
static void Z72_ddN0_ssss_0000_xxxx_0000_0000_segmented(void)
{
	GET_SRC(OP0,NIB3);
	GET_DST(OP0,NIB2);
	GET_IDX(OP1,NIB1);
	WRMEMSEG_B( (RL(dst) + RW(idx)), RB(src) );
}

/******************************************
 ld      rd(rx),rs
 flags:  ------                                                     *
 ******************************************/
static void Z73_ddN0_ssss_0000_xxxx_0000_0000_segmented(void)
{
	GET_SRC(OP0,NIB3);
	GET_DST(OP0,NIB2);
	GET_IDX(OP1,NIB1);
	WRMEMSEG_W( (RL(dst) + RW(idx)), RW(src) );
}

/******************************************
 lda     prd,rs(rx)
 flags:  ------                                                     *
 ******************************************/
static void Z74_ssN0_dddd_0000_xxxx_0000_0000_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_IDX(OP1,NIB1);
	RW(dst) = (RL(src) + RW(idx));
}

/******************************************
 ldl     rrd,rs(rx)
 flags:  ------                                                     *
 ******************************************/
static void Z75_ssN0_dddd_0000_xxxx_0000_0000_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_IDX(OP1,NIB1);
	RL(dst) = RDMEMSEG_L( (RL(src) + RW(idx)) );
}

/******************************************
 lda     prd,addr
 flags:  ------                                                     *
 ******************************************/
static void Z76_0000_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SEGADDR(OP1);
	RL(dst) = seg_addr;
}

/******************************************
 lda     prd,addr(rs)
 flags:  ------                                                     *
 ******************************************/
static void Z76_ssN0_dddd_addr_segmented(void)
{
	GET_DST(OP0,NIB3);
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
	seg_addr += RW(src);
    	RL(dst) = seg_addr;
}

/******************************************
 ldl     rd(rx),rrs
 flags:  ------                                                     *
 ******************************************/
static void Z77_ddN0_ssss_0000_xxxx_0000_0000_segmented(void)
{
	GET_SRC(OP0,NIB3);
	GET_DST(OP0,NIB2);
	GET_IDX(OP1,NIB1);
	WRMEMSEG_L( (RL(dst) + RW(idx)), RL(src) );
}

/******************************************
 ldps    addr
 flags:  CZSVDH                                                     *
 ******************************************/
static void Z79_0000_0000_addr_segmented(void)
{
	UINT16 fcw;
	GET_SEGADDR(OP1);
        Z8001_PRIV_INST_CHECK;

	fcw = RDMEMSEG_W(seg_addr + 2);
	PC  = RDMEMSEG_L(seg_addr + 4);
	CHANGE_FCW(fcw); /* check for user/system mode change */
	CHANGE_PC_PA(seg_addr);
}

/******************************************
 ldps    addr(rs)
 flags:  CZSVDH                                                     *
 ******************************************/
static void Z79_ssN0_0000_addr_segmented(void)
{
	UINT16 fcw;
	GET_SRC(OP0,NIB2);
	GET_SEGADDR(OP1);
        Z8001_PRIV_INST_CHECK;

	seg_addr += RW(src);
	fcw = RDMEMSEG_W(seg_addr + 2);
	PC  = RDMEMSEG_L(seg_addr + 4);
	CHANGE_FCW(fcw); /* check for user/system mode change */
	CHANGE_PC_PA(seg_addr);
}

/******************************************
 iret
 flags:  CZSVDH                                                     *
 ******************************************/
static void Z7B_0000_0000_segmented(void)
{
        UINT32 seg_addr;
	UINT16 tag, fcw;
        Z8001_PRIV_INST_CHECK;

	tag = POPWSEG( SPSEG );	/* get type tag */
	fcw = POPWSEG( SPSEG );	/* get FCW  */
	PC  = POPLSEG( SPSEG );	/* get PC   */
	CHANGE_FCW(fcw);	/* check for user/system mode change */
	CHANGE_PC_PA(seg_addr);
	LOG(("Z8K#%d IRET tag $%04x, fcw $%04x, pc $%04x\n", cpu_getactivecpu(), tag, fcw, PC));
}

/******************************************
 pushl   @rd,rrs
 flags:  ------                                                     *
 ******************************************/
static void Z91_ddN0_ssss_segmented(void)
{
    GET_SRC(OP0,NIB3);
    GET_DST(OP0,NIB2);
    PUSHLSEG( dst, RL(src) );
}

/******************************************
 push    @rd,rs
 flags:  ------                                                     *
 ******************************************/
static void Z93_ddN0_ssss_segmented(void)
{
	GET_SRC(OP0,NIB3);
	GET_DST(OP0,NIB2);
	PUSHWSEG(dst, RW(src));
}

/******************************************
 popl    rrd,@rs
 flags:  ------                                                     *
 ******************************************/
static void Z95_ssN0_dddd_segmented(void)
{
    GET_DST(OP0,NIB3);
    GET_SRC(OP0,NIB2);
    RL(dst) = POPLSEG( src );
}

/******************************************
 pop     rd,@rs
 flags:  ------                                                     *
 ******************************************/
static void Z97_ssN0_dddd_segmented(void)
{
    GET_DST(OP0,NIB3);
    GET_SRC(OP0,NIB2);
    RW(dst) = POPWSEG( src );
}

/******************************************
 ret     cc
 flags:  ------                                                     *
 ******************************************/
static void Z9E_0000_cccc_segmented(void)
{
        UINT32 seg_addr;
	GET_CCC(OP0,NIB3);
	switch (cc) {
		case  0: if (CC0) PC = POPLSEG( SPSEG ); break;
		case  1: if (CC1) PC = POPLSEG( SPSEG ); break;
		case  2: if (CC2) PC = POPLSEG( SPSEG ); break;
		case  3: if (CC3) PC = POPLSEG( SPSEG ); break;
		case  4: if (CC4) PC = POPLSEG( SPSEG ); break;
		case  5: if (CC5) PC = POPLSEG( SPSEG ); break;
		case  6: if (CC6) PC = POPLSEG( SPSEG ); break;
		case  7: if (CC7) PC = POPLSEG( SPSEG ); break;
		case  8: if (CC8) PC = POPLSEG( SPSEG ); break;
		case  9: if (CC9) PC = POPLSEG( SPSEG ); break;
		case 10: if (CCA) PC = POPLSEG( SPSEG ); break;
		case 11: if (CCB) PC = POPLSEG( SPSEG ); break;
		case 12: if (CCC) PC = POPLSEG( SPSEG ); break;
		case 13: if (CCD) PC = POPLSEG( SPSEG ); break;
		case 14: if (CCE) PC = POPLSEG( SPSEG ); break;
		case 15: if (CCF) PC = POPLSEG( SPSEG ); break;
	}
	CHANGE_PC_PA(seg_addr);
}

/******************************************
 trtib   @rd,@rs,rr
 flags:  -ZSV--                                                     *
 ******************************************/
static void ZB8_ddN0_0010_0000_rrrr_ssN0_0000_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SRC(OP1,NIB2);
	GET_CNT(OP1,NIB1);
	UINT8 xlt = RDMEMSEG_B(RL(src) + RDMEMSEG_B(RL(dst)));
	RB(2) = xlt;
	if (xlt) CLR_Z; else SET_Z;
	RW(dst+1)++;   // incr offset, not segment
	if (--RW(cnt)) CLR_V; else SET_V;
}

/******************************************
 trtirb  @rd,@rs,rbr
 flags:  -ZSV--                                                     *
 ******************************************/
static void ZB8_ddN0_0110_0000_rrrr_ssN0_1110_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SRC(OP1,NIB2);
	GET_CNT(OP1,NIB1);
	UINT8 xlt = RDMEMSEG_B(RL(src) + RDMEMSEG_B(RL(dst)));
	RB(2) = xlt;
	if (xlt) CLR_Z; else SET_Z;
	RW(dst+1)++;   // incr offset, not segment
	if (--RW(cnt)) { CLR_V; PC -= 4; } else SET_V;
}

/******************************************
 trtdb   @rd,@rs,rbr
 flags:  -ZSV--                                                     *
 ******************************************/
static void ZB8_ddN0_1010_0000_rrrr_ssN0_0000_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SRC(OP1,NIB2);
	GET_CNT(OP1,NIB1);
	UINT8 xlt = RDMEMSEG_B(RL(src) + RDMEMSEG_B(RL(dst)));
	RB(2) = xlt;
	if (xlt) CLR_Z; else SET_Z;
    	RW(dst+1)--;   // decr offset, not segment
	if (--RW(cnt)) CLR_V; else SET_V;
}

/******************************************
 trtdrb  @rd,@rs,rbr
 flags:  -ZSV--                                                     *
 ******************************************/
static void ZB8_ddN0_1110_0000_rrrr_ssN0_1110_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SRC(OP1,NIB2);
	GET_CNT(OP1,NIB1);
	UINT8 xlt = RDMEMSEG_B(RL(src) + RDMEMSEG_B(RL(dst)));
	RB(2) = xlt;
	if (xlt) CLR_Z; else SET_Z;
    	RW(dst+1)--;   // decr offset, not segment
	if (--RW(cnt)) { CLR_V; PC -= 4; } else SET_V;
}

/******************************************
 trib    @rd,@rs,rbr
 flags:  -ZSV--                                                     *
 ******************************************/
static void ZB8_ddN0_0000_0000_rrrr_ssN0_0000_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SRC(OP1,NIB2);
	GET_CNT(OP1,NIB1);
	UINT8 xlt = RDMEMSEG_B(RL(src) + RDMEMSEG_B(RL(dst)));
	WRMEMSEG_B( RL(dst), xlt );
	RW(dst+1)++;
	if (--RW(cnt)) CLR_V; else SET_V;
}

/******************************************
 trirb   @rd,@rs,rbr
 flags:  -ZSV--                                                     *
 ******************************************/
static void ZB8_ddN0_0100_0000_rrrr_ssN0_0000_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SRC(OP1,NIB2);
	GET_CNT(OP1,NIB1);
	UINT8 xlt = RDMEMSEG_B(RL(src) + RDMEMSEG_B(RL(dst)));
	WRMEMSEG_B( RL(dst), xlt );
	RW(dst+1)++;
	if (--RW(cnt)) { CLR_V; PC -= 4; } else SET_V;
}

/******************************************
 trdb    @rd,@rs,rbr
 flags:  -ZSV--                                                     *
 ******************************************/
static void ZB8_ddN0_1000_0000_rrrr_ssN0_0000_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SRC(OP1,NIB2);
	GET_CNT(OP1,NIB1);
	UINT8 xlt = RDMEMSEG_B(RL(src) + RDMEMSEG_B(RL(dst)));
	WRMEMSEG_B( RL(dst), xlt );
    	RW(dst+1)--;
	if (--RW(cnt)) CLR_V; else SET_V;
}

/******************************************
 trdrb   @rd,@rs,rbr
 flags:  -ZSV--                                                     *
 ******************************************/
static void ZB8_ddN0_1100_0000_rrrr_ssN0_0000_segmented(void)
{
	GET_DST(OP0,NIB2);
	GET_SRC(OP1,NIB2);
	GET_CNT(OP1,NIB1);
	UINT8 xlt = RDMEMSEG_B(RL(src) + RDMEMSEG_B(RL(dst)));
	WRMEMSEG_B( RL(dst), xlt );
    	RW(dst+1)--;
	if (--RW(cnt)) { CLR_V; PC -= 4; } else SET_V;
}

/******************************************
 cpib    rbd,@rs,rr,cc
 flags:  CZSV--                                                     *
 ******************************************/
static void ZBA_ssN0_0000_0000_rrrr_dddd_cccc_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_CCC(OP1,NIB3);
	GET_DST(OP1,NIB2);
	GET_CNT(OP1,NIB1);
	CPB( RB(dst), RDMEMSEG_B(RL(src)) );
	switch (cc) {
		case  0: if (CC0) SET_Z; else CLR_Z; break;
		case  1: if (CC1) SET_Z; else CLR_Z; break;
		case  2: if (CC2) SET_Z; else CLR_Z; break;
		case  3: if (CC3) SET_Z; else CLR_Z; break;
		case  4: if (CC4) SET_Z; else CLR_Z; break;
		case  5: if (CC5) SET_Z; else CLR_Z; break;
		case  6: if (CC6) SET_Z; else CLR_Z; break;
		case  7: if (CC7) SET_Z; else CLR_Z; break;
		case  8: if (CC8) SET_Z; else CLR_Z; break;
		case  9: if (CC9) SET_Z; else CLR_Z; break;
		case 10: if (CCA) SET_Z; else CLR_Z; break;
		case 11: if (CCB) SET_Z; else CLR_Z; break;
		case 12: if (CCC) SET_Z; else CLR_Z; break;
		case 13: if (CCD) SET_Z; else CLR_Z; break;
		case 14: if (CCE) SET_Z; else CLR_Z; break;
		case 15: if (CCF) SET_Z; else CLR_Z; break;
    }
	RW(src+1)++;
	if (--RW(cnt)) CLR_V; else SET_V;
}

/******************************************
 ldib    @rd,@rs,rr
 ldibr   @rd,@rs,rr
 flags:  ---V--                                                     *
 ******************************************/
static void ZBA_ssN0_0001_0000_rrrr_ddN0_x000_segmented(void)
{
	GET_SRC(OP0,NIB2);
    	GET_CNT(OP1,NIB1);
	GET_DST(OP1,NIB2);
	GET_CCC(OP1,NIB3);	/* repeat? */
    	WRMEMSEG_B( RL(dst), RDMEMSEG_B(RL(src)) );
	RW(dst+1)++;
	RW(src+1)++;
	if (--RW(cnt)) { CLR_V; if (cc == 0) PC -= 4; } else SET_V;
}

/******************************************
 cpsib   @rd,@rs,rr,cc
 flags:  CZSV--                                                     *
 ******************************************/
static void ZBA_ssN0_0010_0000_rrrr_ddN0_cccc_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_CCC(OP1,NIB3);
	GET_DST(OP1,NIB2);
	GET_CNT(OP1,NIB1);
	CPB( RDMEMSEG_B(RL(dst)), RDMEMSEG_B(RL(src)) );
	switch (cc) {
		case  0: if (CC0) SET_Z; else CLR_Z; break;
		case  1: if (CC1) SET_Z; else CLR_Z; break;
		case  2: if (CC2) SET_Z; else CLR_Z; break;
		case  3: if (CC3) SET_Z; else CLR_Z; break;
		case  4: if (CC4) SET_Z; else CLR_Z; break;
		case  5: if (CC5) SET_Z; else CLR_Z; break;
		case  6: if (CC6) SET_Z; else CLR_Z; break;
		case  7: if (CC7) SET_Z; else CLR_Z; break;
		case  8: if (CC8) SET_Z; else CLR_Z; break;
		case  9: if (CC9) SET_Z; else CLR_Z; break;
		case 10: if (CCA) SET_Z; else CLR_Z; break;
		case 11: if (CCB) SET_Z; else CLR_Z; break;
		case 12: if (CCC) SET_Z; else CLR_Z; break;
		case 13: if (CCD) SET_Z; else CLR_Z; break;
		case 14: if (CCE) SET_Z; else CLR_Z; break;
		case 15: if (CCF) SET_Z; else CLR_Z; break;
    	}
	RW(dst+1)++;
	RW(src+1)++;
	if (--RW(cnt)) { CLR_V; if (!(FCW & F_Z)) PC -= 4; } else SET_V;
}

/******************************************
 cpirb   rbd,@rs,rr,cc
 flags:  CZSV--                                                     *
 ******************************************/
static void ZBA_ssN0_0100_0000_rrrr_dddd_cccc_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_CCC(OP1,NIB3);
	GET_DST(OP1,NIB2);
	GET_CNT(OP1,NIB1);
	CPB( RB(dst), RDMEMSEG_B(RL(src)) );
	switch (cc) {
		case  0: if (CC0) SET_Z; else CLR_Z; break;
		case  1: if (CC1) SET_Z; else CLR_Z; break;
		case  2: if (CC2) SET_Z; else CLR_Z; break;
		case  3: if (CC3) SET_Z; else CLR_Z; break;
		case  4: if (CC4) SET_Z; else CLR_Z; break;
		case  5: if (CC5) SET_Z; else CLR_Z; break;
		case  6: if (CC6) SET_Z; else CLR_Z; break;
		case  7: if (CC7) SET_Z; else CLR_Z; break;
		case  8: if (CC8) SET_Z; else CLR_Z; break;
		case  9: if (CC9) SET_Z; else CLR_Z; break;
		case 10: if (CCA) SET_Z; else CLR_Z; break;
		case 11: if (CCB) SET_Z; else CLR_Z; break;
		case 12: if (CCC) SET_Z; else CLR_Z; break;
		case 13: if (CCD) SET_Z; else CLR_Z; break;
		case 14: if (CCE) SET_Z; else CLR_Z; break;
		case 15: if (CCF) SET_Z; else CLR_Z; break;
    	}
	RW(src+1)++;
	if (--RW(cnt)) { CLR_V; if (!(FCW & F_Z)) PC -= 4; } else SET_V;
}

/******************************************
 cpsirb  @rd,@rs,rr,cc
 flags:  CZSV--                                                     *
 ******************************************/
static void ZBA_ssN0_0110_0000_rrrr_ddN0_cccc_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_CCC(OP1,NIB3);
	GET_DST(OP1,NIB2);
	GET_CNT(OP1,NIB1);
	CPB( RDMEMSEG_B(RL(dst)), RDMEMSEG_B(RL(src)) );
	switch (cc) {
		case  0: if (CC0) SET_Z; else CLR_Z; break;
		case  1: if (CC1) SET_Z; else CLR_Z; break;
		case  2: if (CC2) SET_Z; else CLR_Z; break;
		case  3: if (CC3) SET_Z; else CLR_Z; break;
		case  4: if (CC4) SET_Z; else CLR_Z; break;
		case  5: if (CC5) SET_Z; else CLR_Z; break;
		case  6: if (CC6) SET_Z; else CLR_Z; break;
		case  7: if (CC7) SET_Z; else CLR_Z; break;
		case  8: if (CC8) SET_Z; else CLR_Z; break;
		case  9: if (CC9) SET_Z; else CLR_Z; break;
		case 10: if (CCA) SET_Z; else CLR_Z; break;
		case 11: if (CCB) SET_Z; else CLR_Z; break;
		case 12: if (CCC) SET_Z; else CLR_Z; break;
		case 13: if (CCD) SET_Z; else CLR_Z; break;
		case 14: if (CCE) SET_Z; else CLR_Z; break;
		case 15: if (CCF) SET_Z; else CLR_Z; break;
    }
	RW(dst+1)++;
	RW(src+1)++;
	if (--RW(cnt)) { CLR_V; if (!(FCW & F_Z)) PC -= 4; } else SET_V;
}

/******************************************
 cpdb    rbd,@rs,rr,cc
 flags:  CZSV--                                                     *
 ******************************************/
static void ZBA_ssN0_1000_0000_rrrr_dddd_cccc_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_CCC(OP1,NIB3);
	GET_DST(OP1,NIB2);
	GET_CNT(OP1,NIB1);
    	CPB( RB(dst), RDMEMSEG_B(RL(src)) );
    	switch (cc) {
		case  0: if (CC0) SET_Z; else CLR_Z; break;
		case  1: if (CC1) SET_Z; else CLR_Z; break;
		case  2: if (CC2) SET_Z; else CLR_Z; break;
		case  3: if (CC3) SET_Z; else CLR_Z; break;
		case  4: if (CC4) SET_Z; else CLR_Z; break;
		case  5: if (CC5) SET_Z; else CLR_Z; break;
		case  6: if (CC6) SET_Z; else CLR_Z; break;
		case  7: if (CC7) SET_Z; else CLR_Z; break;
		case  8: if (CC8) SET_Z; else CLR_Z; break;
		case  9: if (CC9) SET_Z; else CLR_Z; break;
		case 10: if (CCA) SET_Z; else CLR_Z; break;
		case 11: if (CCB) SET_Z; else CLR_Z; break;
		case 12: if (CCC) SET_Z; else CLR_Z; break;
		case 13: if (CCD) SET_Z; else CLR_Z; break;
		case 14: if (CCE) SET_Z; else CLR_Z; break;
		case 15: if (CCF) SET_Z; else CLR_Z; break;
    	}
    	RW(src+1)--;
	if (--RW(cnt)) CLR_V; else SET_V;
}

/******************************************
 lddb    @rs,@rd,rr
 lddbr   @rs,@rd,rr
 flags:  ---V--                                                     *
 ******************************************/
static void ZBA_ssN0_1001_0000_rrrr_ddN0_x000_segmented(void)
{
	GET_SRC(OP0,NIB2);
    	GET_CNT(OP1,NIB1);
	GET_DST(OP1,NIB2);
	GET_CCC(OP1,NIB3);
	WRMEMSEG_B( RL(dst), RDMEMSEG_B(RL(src)) );
	RW(dst+1)--;
	RW(src+1)--;
	if (--RW(cnt)) { CLR_V; if (cc == 0) PC -= 4; } else SET_V;
}

/******************************************
 cpsdb   @rd,@rs,rr,cc
 flags:  CZSV--                                                     *
 ******************************************/
static void ZBA_ssN0_1010_0000_rrrr_ddN0_cccc_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_CCC(OP1,NIB3);
	GET_DST(OP1,NIB2);
	GET_CNT(OP1,NIB1);
    	CPB( RDMEMSEG_B(RL(dst)), RDMEMSEG_B(RL(src)) );
	switch (cc) {
		case  0: if (CC0) SET_Z; else CLR_Z; break;
		case  1: if (CC1) SET_Z; else CLR_Z; break;
		case  2: if (CC2) SET_Z; else CLR_Z; break;
		case  3: if (CC3) SET_Z; else CLR_Z; break;
		case  4: if (CC4) SET_Z; else CLR_Z; break;
		case  5: if (CC5) SET_Z; else CLR_Z; break;
		case  6: if (CC6) SET_Z; else CLR_Z; break;
		case  7: if (CC7) SET_Z; else CLR_Z; break;
		case  8: if (CC8) SET_Z; else CLR_Z; break;
		case  9: if (CC9) SET_Z; else CLR_Z; break;
		case 10: if (CCA) SET_Z; else CLR_Z; break;
		case 11: if (CCB) SET_Z; else CLR_Z; break;
		case 12: if (CCC) SET_Z; else CLR_Z; break;
		case 13: if (CCD) SET_Z; else CLR_Z; break;
		case 14: if (CCE) SET_Z; else CLR_Z; break;
		case 15: if (CCF) SET_Z; else CLR_Z; break;
    	}
	RW(dst+1)--;
	RW(src+1)--;
	if (--RW(cnt)) CLR_V; else SET_V;
}

/******************************************
 cpdrb   rbd,@rs,rr,cc
 flags:  CZSV--                                                     *
 ******************************************/
static void ZBA_ssN0_1100_0000_rrrr_dddd_cccc_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_CCC(OP1,NIB3);
	GET_DST(OP1,NIB2);
	GET_CNT(OP1,NIB1);
	CPB( RB(dst), RDMEMSEG_B(RL(src)) );
	switch (cc) {
		case  0: if (CC0) SET_Z; else CLR_Z; break;
		case  1: if (CC1) SET_Z; else CLR_Z; break;
		case  2: if (CC2) SET_Z; else CLR_Z; break;
		case  3: if (CC3) SET_Z; else CLR_Z; break;
		case  4: if (CC4) SET_Z; else CLR_Z; break;
		case  5: if (CC5) SET_Z; else CLR_Z; break;
		case  6: if (CC6) SET_Z; else CLR_Z; break;
		case  7: if (CC7) SET_Z; else CLR_Z; break;
		case  8: if (CC8) SET_Z; else CLR_Z; break;
		case  9: if (CC9) SET_Z; else CLR_Z; break;
		case 10: if (CCA) SET_Z; else CLR_Z; break;
		case 11: if (CCB) SET_Z; else CLR_Z; break;
		case 12: if (CCC) SET_Z; else CLR_Z; break;
		case 13: if (CCD) SET_Z; else CLR_Z; break;
		case 14: if (CCE) SET_Z; else CLR_Z; break;
		case 15: if (CCF) SET_Z; else CLR_Z; break;
    }
    RW(src+1)--;
	if (--RW(cnt)) { CLR_V; if (!(FCW & F_Z)) PC -= 4; } else SET_V;
}

/******************************************
 cpsdrb  @rd,@rs,rr,cc
 flags:  CZSV--                                                     *
 ******************************************/
static void ZBA_ssN0_1110_0000_rrrr_ddN0_cccc_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_CCC(OP1,NIB3);
	GET_DST(OP1,NIB2);
	GET_CNT(OP1,NIB1);
    CPB( RDMEMSEG_B(RL(dst)), RDMEMSEG_B(RL(src)) );
	switch (cc) {
		case  0: if (CC0) SET_Z; else CLR_Z; break;
		case  1: if (CC1) SET_Z; else CLR_Z; break;
		case  2: if (CC2) SET_Z; else CLR_Z; break;
		case  3: if (CC3) SET_Z; else CLR_Z; break;
		case  4: if (CC4) SET_Z; else CLR_Z; break;
		case  5: if (CC5) SET_Z; else CLR_Z; break;
		case  6: if (CC6) SET_Z; else CLR_Z; break;
		case  7: if (CC7) SET_Z; else CLR_Z; break;
		case  8: if (CC8) SET_Z; else CLR_Z; break;
		case  9: if (CC9) SET_Z; else CLR_Z; break;
		case 10: if (CCA) SET_Z; else CLR_Z; break;
		case 11: if (CCB) SET_Z; else CLR_Z; break;
		case 12: if (CCC) SET_Z; else CLR_Z; break;
		case 13: if (CCD) SET_Z; else CLR_Z; break;
		case 14: if (CCE) SET_Z; else CLR_Z; break;
		case 15: if (CCF) SET_Z; else CLR_Z; break;
	}
	RW(dst+1)--;
	RW(src+1)--;
	if (--RW(cnt)) { CLR_V; if (!(FCW & F_Z)) PC -= 4; } else SET_V;
}

/******************************************
 cpi     rd,@rs,rr,cc
 flags:  CZSV--                                                     *
 ******************************************/
static void ZBB_ssN0_0000_0000_rrrr_dddd_cccc_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_CCC(OP1,NIB3);
	GET_DST(OP1,NIB2);
	GET_CNT(OP1,NIB1);
	CPW( RW(dst), RDMEMSEG_W(RL(src)) );
    	switch (cc) {
		case  0: if (CC0) SET_Z; else CLR_Z; break;
		case  1: if (CC1) SET_Z; else CLR_Z; break;
		case  2: if (CC2) SET_Z; else CLR_Z; break;
		case  3: if (CC3) SET_Z; else CLR_Z; break;
		case  4: if (CC4) SET_Z; else CLR_Z; break;
		case  5: if (CC5) SET_Z; else CLR_Z; break;
		case  6: if (CC6) SET_Z; else CLR_Z; break;
		case  7: if (CC7) SET_Z; else CLR_Z; break;
		case  8: if (CC8) SET_Z; else CLR_Z; break;
		case  9: if (CC9) SET_Z; else CLR_Z; break;
		case 10: if (CCA) SET_Z; else CLR_Z; break;
		case 11: if (CCB) SET_Z; else CLR_Z; break;
		case 12: if (CCC) SET_Z; else CLR_Z; break;
		case 13: if (CCD) SET_Z; else CLR_Z; break;
		case 14: if (CCE) SET_Z; else CLR_Z; break;
		case 15: if (CCF) SET_Z; else CLR_Z; break;
    }
    RW(src+1) += 2;
	if (--RW(cnt)) CLR_V; else SET_V;
}

/******************************************
 ldi     @rd,@rs,rr
 ldir    @rd,@rs,rr
 flags:  ---V--                                                     *
 ******************************************/
static void ZBB_ssN0_0001_0000_rrrr_ddN0_x000_segmented(void)
{
	GET_SRC(OP0,NIB2);
    GET_CNT(OP1,NIB1);
	GET_DST(OP1,NIB2);
	GET_CCC(OP1,NIB3);
	WRMEMSEG_W( RL(dst), RDMEMSEG_W(RL(src)) );
	RW(dst+1) += 2;
	RW(src+1) += 2;
	if (--RW(cnt)) { CLR_V; if (cc == 0) PC -= 4; } else SET_V;
}

/******************************************
 cpsi    @rd,@rs,rr,cc
 flags:  CZSV--                                                     *
 ******************************************/
static void ZBB_ssN0_0010_0000_rrrr_ddN0_cccc_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_CCC(OP1,NIB3);
	GET_DST(OP1,NIB2);
	GET_CNT(OP1,NIB1);
	CPW( RDMEMSEG_W(RL(dst)), RDMEMSEG_W(RL(src)) );
	switch (cc) {
		case  0: if (CC0) SET_Z; else CLR_Z; break;
		case  1: if (CC1) SET_Z; else CLR_Z; break;
		case  2: if (CC2) SET_Z; else CLR_Z; break;
		case  3: if (CC3) SET_Z; else CLR_Z; break;
		case  4: if (CC4) SET_Z; else CLR_Z; break;
		case  5: if (CC5) SET_Z; else CLR_Z; break;
		case  6: if (CC6) SET_Z; else CLR_Z; break;
		case  7: if (CC7) SET_Z; else CLR_Z; break;
		case  8: if (CC8) SET_Z; else CLR_Z; break;
		case  9: if (CC9) SET_Z; else CLR_Z; break;
		case 10: if (CCA) SET_Z; else CLR_Z; break;
		case 11: if (CCB) SET_Z; else CLR_Z; break;
		case 12: if (CCC) SET_Z; else CLR_Z; break;
		case 13: if (CCD) SET_Z; else CLR_Z; break;
		case 14: if (CCE) SET_Z; else CLR_Z; break;
		case 15: if (CCF) SET_Z; else CLR_Z; break;
    }
	RW(dst+1) += 2;
	RW(src+1) += 2;
	if (--RW(cnt)) CLR_V; else SET_V;
}

/******************************************
 cpir    rd,@rs,rr,cc
 flags:  CZSV--                                                     *
 ******************************************/
static void ZBB_ssN0_0100_0000_rrrr_dddd_cccc_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_CCC(OP1,NIB3);
	GET_DST(OP1,NIB2);
	GET_CNT(OP1,NIB1);
	CPW( RW(dst), RDMEMSEG_W(RL(src)) );
	switch (cc) {
		case  0: if (CC0) SET_Z; else CLR_Z; break;
		case  1: if (CC1) SET_Z; else CLR_Z; break;
		case  2: if (CC2) SET_Z; else CLR_Z; break;
		case  3: if (CC3) SET_Z; else CLR_Z; break;
		case  4: if (CC4) SET_Z; else CLR_Z; break;
		case  5: if (CC5) SET_Z; else CLR_Z; break;
		case  6: if (CC6) SET_Z; else CLR_Z; break;
		case  7: if (CC7) SET_Z; else CLR_Z; break;
		case  8: if (CC8) SET_Z; else CLR_Z; break;
		case  9: if (CC9) SET_Z; else CLR_Z; break;
		case 10: if (CCA) SET_Z; else CLR_Z; break;
		case 11: if (CCB) SET_Z; else CLR_Z; break;
		case 12: if (CCC) SET_Z; else CLR_Z; break;
		case 13: if (CCD) SET_Z; else CLR_Z; break;
		case 14: if (CCE) SET_Z; else CLR_Z; break;
		case 15: if (CCF) SET_Z; else CLR_Z; break;
    }
	RW(src+1) += 2;
	if (--RW(cnt)) { CLR_V; if (!(FCW & F_Z)) PC -= 4; } else SET_V;
}

/******************************************
 cpsir   @rd,@rs,rr,cc
 flags:  CZSV--                                                     *
 ******************************************/
static void ZBB_ssN0_0110_0000_rrrr_ddN0_cccc_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_CCC(OP1,NIB3);
	GET_DST(OP1,NIB2);
	GET_CNT(OP1,NIB1);
	CPW( RDMEMSEG_W(RL(dst)), RDMEMSEG_W(RL(src)) );
	switch (cc) {
		case  0: if (CC0) SET_Z; else CLR_Z; break;
		case  1: if (CC1) SET_Z; else CLR_Z; break;
		case  2: if (CC2) SET_Z; else CLR_Z; break;
		case  3: if (CC3) SET_Z; else CLR_Z; break;
		case  4: if (CC4) SET_Z; else CLR_Z; break;
		case  5: if (CC5) SET_Z; else CLR_Z; break;
		case  6: if (CC6) SET_Z; else CLR_Z; break;
		case  7: if (CC7) SET_Z; else CLR_Z; break;
		case  8: if (CC8) SET_Z; else CLR_Z; break;
		case  9: if (CC9) SET_Z; else CLR_Z; break;
		case 10: if (CCA) SET_Z; else CLR_Z; break;
		case 11: if (CCB) SET_Z; else CLR_Z; break;
		case 12: if (CCC) SET_Z; else CLR_Z; break;
		case 13: if (CCD) SET_Z; else CLR_Z; break;
		case 14: if (CCE) SET_Z; else CLR_Z; break;
		case 15: if (CCF) SET_Z; else CLR_Z; break;
    	}
	RW(dst+1) += 2;
    	RW(src+1) += 2;
	if (--RW(cnt)) { CLR_V; if (!(FCW & F_Z)) PC -= 4; } else SET_V;
}

/******************************************
 cpd     rd,@rs,rr,cc
 flags:  CZSV--                                                     *
 ******************************************/
static void ZBB_ssN0_1000_0000_rrrr_dddd_cccc_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_CCC(OP1,NIB3);
	GET_DST(OP1,NIB2);
	GET_CNT(OP1,NIB1);
	CPW( RW(dst), RDMEMSEG_W(RL(src)) );
    	switch (cc) {
		case  0: if (CC0) SET_Z; else CLR_Z; break;
		case  1: if (CC1) SET_Z; else CLR_Z; break;
		case  2: if (CC2) SET_Z; else CLR_Z; break;
		case  3: if (CC3) SET_Z; else CLR_Z; break;
		case  4: if (CC4) SET_Z; else CLR_Z; break;
		case  5: if (CC5) SET_Z; else CLR_Z; break;
		case  6: if (CC6) SET_Z; else CLR_Z; break;
		case  7: if (CC7) SET_Z; else CLR_Z; break;
		case  8: if (CC8) SET_Z; else CLR_Z; break;
		case  9: if (CC9) SET_Z; else CLR_Z; break;
		case 10: if (CCA) SET_Z; else CLR_Z; break;
		case 11: if (CCB) SET_Z; else CLR_Z; break;
		case 12: if (CCC) SET_Z; else CLR_Z; break;
		case 13: if (CCD) SET_Z; else CLR_Z; break;
		case 14: if (CCE) SET_Z; else CLR_Z; break;
		case 15: if (CCF) SET_Z; else CLR_Z; break;
    	}
    	RW(src+1) -= 2;
	if (--RW(cnt)) CLR_V; else SET_V;
}

/******************************************
 ldd     @rs,@rd,rr
 lddr    @rs,@rd,rr
 flags:  ---V--                                                     *
 ******************************************/
static void ZBB_ssN0_1001_0000_rrrr_ddN0_x000_segmented(void)
{
    	GET_SRC(OP0,NIB2);
    	GET_CNT(OP1,NIB1);
    	GET_DST(OP1,NIB2);
	GET_CCC(OP1,NIB3);
    	WRMEMSEG_W( RL(dst), RDMEMSEG_W(RL(src)) );
    	RW(dst+1) -= 2;
    	RW(src+1) -= 2;
	if (--RW(cnt)) { CLR_V; if (cc == 0) PC -= 4; } else SET_V;
}

/******************************************
 cpsd    @rd,@rs,rr,cc
 flags:  CZSV--                                                     *
 ******************************************/
static void ZBB_ssN0_1010_0000_rrrr_ddN0_cccc_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_CCC(OP1,NIB3);
	GET_DST(OP1,NIB2);
	GET_CNT(OP1,NIB1);
	CPW( RDMEMSEG_W(RL(dst)), RDMEMSEG_W(RL(src)) );
	switch (cc) {
		case  0: if (CC0) SET_Z; else CLR_Z; break;
		case  1: if (CC1) SET_Z; else CLR_Z; break;
		case  2: if (CC2) SET_Z; else CLR_Z; break;
		case  3: if (CC3) SET_Z; else CLR_Z; break;
		case  4: if (CC4) SET_Z; else CLR_Z; break;
		case  5: if (CC5) SET_Z; else CLR_Z; break;
		case  6: if (CC6) SET_Z; else CLR_Z; break;
		case  7: if (CC7) SET_Z; else CLR_Z; break;
		case  8: if (CC8) SET_Z; else CLR_Z; break;
		case  9: if (CC9) SET_Z; else CLR_Z; break;
		case 10: if (CCA) SET_Z; else CLR_Z; break;
		case 11: if (CCB) SET_Z; else CLR_Z; break;
		case 12: if (CCC) SET_Z; else CLR_Z; break;
		case 13: if (CCD) SET_Z; else CLR_Z; break;
		case 14: if (CCE) SET_Z; else CLR_Z; break;
		case 15: if (CCF) SET_Z; else CLR_Z; break;
    	}
    	RW(dst+1) -= 2;
    	RW(src+1) -= 2;
	if (--RW(cnt)) CLR_V; else SET_V;
}

/******************************************
 cpdr    rd,@rs,rr,cc
 flags:  CZSV--                                                     *
 ******************************************/
static void ZBB_ssN0_1100_0000_rrrr_dddd_cccc_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_CCC(OP1,NIB3);
	GET_DST(OP1,NIB2);
	GET_CNT(OP1,NIB1);
	CPW( RW(dst), RDMEMSEG_W(RL(src)) );
	switch (cc) {
		case  0: if (CC0) SET_Z; else CLR_Z; break;
		case  1: if (CC1) SET_Z; else CLR_Z; break;
		case  2: if (CC2) SET_Z; else CLR_Z; break;
		case  3: if (CC3) SET_Z; else CLR_Z; break;
		case  4: if (CC4) SET_Z; else CLR_Z; break;
		case  5: if (CC5) SET_Z; else CLR_Z; break;
		case  6: if (CC6) SET_Z; else CLR_Z; break;
		case  7: if (CC7) SET_Z; else CLR_Z; break;
		case  8: if (CC8) SET_Z; else CLR_Z; break;
		case  9: if (CC9) SET_Z; else CLR_Z; break;
		case 10: if (CCA) SET_Z; else CLR_Z; break;
		case 11: if (CCB) SET_Z; else CLR_Z; break;
		case 12: if (CCC) SET_Z; else CLR_Z; break;
		case 13: if (CCD) SET_Z; else CLR_Z; break;
		case 14: if (CCE) SET_Z; else CLR_Z; break;
		case 15: if (CCF) SET_Z; else CLR_Z; break;
    	}
    	RW(src+1) -= 2;
	if (--RW(cnt)) { CLR_V; if (!(FCW & F_Z)) PC -= 4; } else SET_V;
}

/******************************************
 cpsdr   @rd,@rs,rr,cc
 flags:  CZSV--                                                     *
 ******************************************/
static void ZBB_ssN0_1110_0000_rrrr_ddN0_cccc_segmented(void)
{
	GET_SRC(OP0,NIB2);
	GET_CCC(OP1,NIB3);
	GET_DST(OP1,NIB2);
	GET_CNT(OP1,NIB1);
	CPW( RDMEMSEG_W(RL(dst)), RDMEMSEG_W(RL(src)) );
	switch (cc) {
		case  0: if (CC0) SET_Z; else CLR_Z; break;
		case  1: if (CC1) SET_Z; else CLR_Z; break;
		case  2: if (CC2) SET_Z; else CLR_Z; break;
		case  3: if (CC3) SET_Z; else CLR_Z; break;
		case  4: if (CC4) SET_Z; else CLR_Z; break;
		case  5: if (CC5) SET_Z; else CLR_Z; break;
		case  6: if (CC6) SET_Z; else CLR_Z; break;
		case  7: if (CC7) SET_Z; else CLR_Z; break;
		case  8: if (CC8) SET_Z; else CLR_Z; break;
		case  9: if (CC9) SET_Z; else CLR_Z; break;
		case 10: if (CCA) SET_Z; else CLR_Z; break;
		case 11: if (CCB) SET_Z; else CLR_Z; break;
		case 12: if (CCC) SET_Z; else CLR_Z; break;
		case 13: if (CCD) SET_Z; else CLR_Z; break;
		case 14: if (CCE) SET_Z; else CLR_Z; break;
		case 15: if (CCF) SET_Z; else CLR_Z; break;
    	}
    	RW(dst+1) -= 2;
    	RW(src+1) -= 2;
	if (--RW(cnt)) { CLR_V; if (!(FCW & F_Z)) PC -= 4; } else SET_V;
}

/******************************************
 calr    dsp12
 flags:  ------                                                     *
 ******************************************/
static void ZD_dsp12_segmented(void)
{
        UINT32 seg_addr;
	INT16 dsp12 = Z.op[0] & 0xfff;
	PUSHLSEG( SPSEG, PC );
	dsp12 = (dsp12 & 2048) ? 4096 -2 * (dsp12 & 2047) : -2 * (dsp12 & 2047);
	PC += dsp12;
	CHANGE_PC_PA(seg_addr);
}


